# Copyright (c) 2015 Amazon.com, Inc. or its affiliates.  All Rights Reserved
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#

import boto
from boto.compat import json, urlsplit
from boto.connection import AWSQueryConnection
from boto.regioninfo import RegionInfo
from boto.exception import JSONResponseError
from boto.machinelearning import exceptions


class MachineLearningConnection(AWSQueryConnection):
    """
    Definition of the public APIs exposed by Amazon Machine Learning
    """
    APIVersion = "2014-12-12"
    AuthServiceName = 'machinelearning'
    DefaultRegionName = "us-east-1"
    DefaultRegionEndpoint = "machinelearning.us-east-1.amazonaws.com"
    ServiceName = "MachineLearning"
    TargetPrefix = "AmazonML_20141212"
    ResponseError = JSONResponseError

    _faults = {
        "InternalServerException": exceptions.InternalServerException,
        "LimitExceededException": exceptions.LimitExceededException,
        "ResourceNotFoundException": exceptions.ResourceNotFoundException,
        "IdempotentParameterMismatchException": exceptions.IdempotentParameterMismatchException,
        "PredictorNotMountedException": exceptions.PredictorNotMountedException,
        "InvalidInputException": exceptions.InvalidInputException,
    }


    def __init__(self, **kwargs):
        region = kwargs.pop('region', None)
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionEndpoint)

        if 'host' not in kwargs or kwargs['host'] is None:
            kwargs['host'] = region.endpoint

        super(MachineLearningConnection, self).__init__(**kwargs)
        self.region = region
        self.auth_region_name = self.region.name

    def _required_auth_capability(self):
        return ['hmac-v4']

    def create_batch_prediction(self, batch_prediction_id, ml_model_id,
                                batch_prediction_data_source_id, output_uri,
                                batch_prediction_name=None):
        """
        Generates predictions for a group of observations. The
        observations to process exist in one or more data files
        referenced by a `DataSource`. This operation creates a new
        `BatchPrediction`, and uses an `MLModel` and the data files
        referenced by the `DataSource` as information sources.

        `CreateBatchPrediction` is an asynchronous operation. In
        response to `CreateBatchPrediction`, Amazon Machine Learning
        (Amazon ML) immediately returns and sets the `BatchPrediction`
        status to `PENDING`. After the `BatchPrediction` completes,
        Amazon ML sets the status to `COMPLETED`.

        You can poll for status updates by using the
        GetBatchPrediction operation and checking the `Status`
        parameter of the result. After the `COMPLETED` status appears,
        the results are available in the location specified by the
        `OutputUri` parameter.

        :type batch_prediction_id: string
        :param batch_prediction_id: A user-supplied ID that uniquely identifies
            the `BatchPrediction`.

        :type batch_prediction_name: string
        :param batch_prediction_name: A user-supplied name or description of
            the `BatchPrediction`. `BatchPredictionName` can only use the UTF-8
            character set.

        :type ml_model_id: string
        :param ml_model_id: The ID of the `MLModel` that will generate
            predictions for the group of observations.

        :type batch_prediction_data_source_id: string
        :param batch_prediction_data_source_id: The ID of the `DataSource` that
            points to the group of observations to predict.

        :type output_uri: string
        :param output_uri: The location of an Amazon Simple Storage Service
            (Amazon S3) bucket or directory to store the batch prediction
            results. The following substrings are not allowed in the s3 key
            portion of the "outputURI" field: ':', '//', '/./', '/../'.
        Amazon ML needs permissions to store and retrieve the logs on your
            behalf. For information about how to set permissions, see the
            `Amazon Machine Learning Developer Guide`_.

        """
        params = {
            'BatchPredictionId': batch_prediction_id,
            'MLModelId': ml_model_id,
            'BatchPredictionDataSourceId': batch_prediction_data_source_id,
            'OutputUri': output_uri,
        }
        if batch_prediction_name is not None:
            params['BatchPredictionName'] = batch_prediction_name
        return self.make_request(action='CreateBatchPrediction',
                                 body=json.dumps(params))

    def create_data_source_from_rds(self, data_source_id, rds_data, role_arn,
                                    data_source_name=None,
                                    compute_statistics=None):
        """
        Creates a `DataSource` object from an ` Amazon Relational
        Database Service`_ (Amazon RDS). A `DataSource` references
        data that can be used to perform CreateMLModel,
        CreateEvaluation, or CreateBatchPrediction operations.

        `CreateDataSourceFromRDS` is an asynchronous operation. In
        response to `CreateDataSourceFromRDS`, Amazon Machine Learning
        (Amazon ML) immediately returns and sets the `DataSource`
        status to `PENDING`. After the `DataSource` is created and
        ready for use, Amazon ML sets the `Status` parameter to
        `COMPLETED`. `DataSource` in `COMPLETED` or `PENDING` status
        can only be used to perform CreateMLModel, CreateEvaluation,
        or CreateBatchPrediction operations.

        If Amazon ML cannot accept the input source, it sets the
        `Status` parameter to `FAILED` and includes an error message
        in the `Message` attribute of the GetDataSource operation
        response.

        :type data_source_id: string
        :param data_source_id: A user-supplied ID that uniquely identifies the
            `DataSource`. Typically, an Amazon Resource Number (ARN) becomes
            the ID for a `DataSource`.

        :type data_source_name: string
        :param data_source_name: A user-supplied name or description of the
            `DataSource`.

        :type rds_data: dict
        :param rds_data:
        The data specification of an Amazon RDS `DataSource`:


        + DatabaseInformation -

            + `DatabaseName ` - Name of the Amazon RDS database.
            + ` InstanceIdentifier ` - Unique identifier for the Amazon RDS
                  database instance.

        + DatabaseCredentials - AWS Identity and Access Management (IAM)
              credentials that are used to connect to the Amazon RDS database.
        + ResourceRole - Role (DataPipelineDefaultResourceRole) assumed by an
              Amazon Elastic Compute Cloud (EC2) instance to carry out the copy
              task from Amazon RDS to Amazon S3. For more information, see `Role
              templates`_ for data pipelines.
        + ServiceRole - Role (DataPipelineDefaultRole) assumed by the AWS Data
              Pipeline service to monitor the progress of the copy task from
              Amazon RDS to Amazon Simple Storage Service (S3). For more
              information, see `Role templates`_ for data pipelines.
        + SecurityInfo - Security information to use to access an Amazon RDS
              instance. You need to set up appropriate ingress rules for the
              security entity IDs provided to allow access to the Amazon RDS
              instance. Specify a [ `SubnetId`, `SecurityGroupIds`] pair for a
              VPC-based Amazon RDS instance.
        + SelectSqlQuery - Query that is used to retrieve the observation data
              for the `Datasource`.
        + S3StagingLocation - Amazon S3 location for staging RDS data. The data
              retrieved from Amazon RDS using `SelectSqlQuery` is stored in this
              location.
        + DataSchemaUri - Amazon S3 location of the `DataSchema`.
        + DataSchema - A JSON string representing the schema. This is not
              required if `DataSchemaUri` is specified.
        + DataRearrangement - A JSON string representing the splitting
              requirement of a `Datasource`. Sample - ` "{\"randomSeed\":\"some-
              random-seed\",
              \"splitting\":{\"percentBegin\":10,\"percentEnd\":60}}"`

        :type role_arn: string
        :param role_arn: The role that Amazon ML assumes on behalf of the user
            to create and activate a data pipeline in the users account and
            copy data (using the `SelectSqlQuery`) query from Amazon RDS to
            Amazon S3.

        :type compute_statistics: boolean
        :param compute_statistics: The compute statistics for a `DataSource`.
            The statistics are generated from the observation data referenced
            by a `DataSource`. Amazon ML uses the statistics internally during
            an `MLModel` training. This parameter must be set to `True` if the
            ``DataSource `` needs to be used for `MLModel` training.

        """
        params = {
            'DataSourceId': data_source_id,
            'RDSData': rds_data,
            'RoleARN': role_arn,
        }
        if data_source_name is not None:
            params['DataSourceName'] = data_source_name
        if compute_statistics is not None:
            params['ComputeStatistics'] = compute_statistics
        return self.make_request(action='CreateDataSourceFromRDS',
                                 body=json.dumps(params))

    def create_data_source_from_redshift(self, data_source_id, data_spec,
                                         role_arn, data_source_name=None,
                                         compute_statistics=None):
        """
        Creates a `DataSource` from `Amazon Redshift`_. A `DataSource`
        references data that can be used to perform either
        CreateMLModel, CreateEvaluation or CreateBatchPrediction
        operations.

        `CreateDataSourceFromRedshift` is an asynchronous operation.
        In response to `CreateDataSourceFromRedshift`, Amazon Machine
        Learning (Amazon ML) immediately returns and sets the
        `DataSource` status to `PENDING`. After the `DataSource` is
        created and ready for use, Amazon ML sets the `Status`
        parameter to `COMPLETED`. `DataSource` in `COMPLETED` or
        `PENDING` status can only be used to perform CreateMLModel,
        CreateEvaluation, or CreateBatchPrediction operations.

        If Amazon ML cannot accept the input source, it sets the
        `Status` parameter to `FAILED` and includes an error message
        in the `Message` attribute of the GetDataSource operation
        response.

        The observations should exist in the database hosted on an
        Amazon Redshift cluster and should be specified by a
        `SelectSqlQuery`. Amazon ML executes ` Unload`_ command in
        Amazon Redshift to transfer the result set of `SelectSqlQuery`
        to `S3StagingLocation.`

        After the `DataSource` is created, it's ready for use in
        evaluations and batch predictions. If you plan to use the
        `DataSource` to train an `MLModel`, the `DataSource` requires
        another item -- a recipe. A recipe describes the observation
        variables that participate in training an `MLModel`. A recipe
        describes how each input variable will be used in training.
        Will the variable be included or excluded from training? Will
        the variable be manipulated, for example, combined with
        another variable or split apart into word combinations? The
        recipe provides answers to these questions. For more
        information, see the Amazon Machine Learning Developer Guide.

        :type data_source_id: string
        :param data_source_id: A user-supplied ID that uniquely identifies the
            `DataSource`.

        :type data_source_name: string
        :param data_source_name: A user-supplied name or description of the
            `DataSource`.

        :type data_spec: dict
        :param data_spec:
        The data specification of an Amazon Redshift `DataSource`:


        + DatabaseInformation -

            + `DatabaseName ` - Name of the Amazon Redshift database.
            + ` ClusterIdentifier ` - Unique ID for the Amazon Redshift cluster.

        + DatabaseCredentials - AWS Identity abd Access Management (IAM)
              credentials that are used to connect to the Amazon Redshift
              database.
        + SelectSqlQuery - Query that is used to retrieve the observation data
              for the `Datasource`.
        + S3StagingLocation - Amazon Simple Storage Service (Amazon S3)
              location for staging Amazon Redshift data. The data retrieved from
              Amazon Relational Database Service (Amazon RDS) using
              `SelectSqlQuery` is stored in this location.
        + DataSchemaUri - Amazon S3 location of the `DataSchema`.
        + DataSchema - A JSON string representing the schema. This is not
              required if `DataSchemaUri` is specified.
        + DataRearrangement - A JSON string representing the splitting
              requirement of a `Datasource`. Sample - ` "{\"randomSeed\":\"some-
              random-seed\",
              \"splitting\":{\"percentBegin\":10,\"percentEnd\":60}}"`

        :type role_arn: string
        :param role_arn: A fully specified role Amazon Resource Name (ARN).
            Amazon ML assumes the role on behalf of the user to create the
            following:


        + A security group to allow Amazon ML to execute the `SelectSqlQuery`
              query on an Amazon Redshift cluster
        + An Amazon S3 bucket policy to grant Amazon ML read/write permissions
              on the `S3StagingLocation`

        :type compute_statistics: boolean
        :param compute_statistics: The compute statistics for a `DataSource`.
            The statistics are generated from the observation data referenced
            by a `DataSource`. Amazon ML uses the statistics internally during
            `MLModel` training. This parameter must be set to `True` if the
            ``DataSource `` needs to be used for `MLModel` training

        """
        params = {
            'DataSourceId': data_source_id,
            'DataSpec': data_spec,
            'RoleARN': role_arn,
        }
        if data_source_name is not None:
            params['DataSourceName'] = data_source_name
        if compute_statistics is not None:
            params['ComputeStatistics'] = compute_statistics
        return self.make_request(action='CreateDataSourceFromRedshift',
                                 body=json.dumps(params))

    def create_data_source_from_s3(self, data_source_id, data_spec,
                                   data_source_name=None,
                                   compute_statistics=None):
        """
        Creates a `DataSource` object. A `DataSource` references data
        that can be used to perform CreateMLModel, CreateEvaluation,
        or CreateBatchPrediction operations.

        `CreateDataSourceFromS3` is an asynchronous operation. In
        response to `CreateDataSourceFromS3`, Amazon Machine Learning
        (Amazon ML) immediately returns and sets the `DataSource`
        status to `PENDING`. After the `DataSource` is created and
        ready for use, Amazon ML sets the `Status` parameter to
        `COMPLETED`. `DataSource` in `COMPLETED` or `PENDING` status
        can only be used to perform CreateMLModel, CreateEvaluation or
        CreateBatchPrediction operations.

        If Amazon ML cannot accept the input source, it sets the
        `Status` parameter to `FAILED` and includes an error message
        in the `Message` attribute of the GetDataSource operation
        response.

        The observation data used in a `DataSource` should be ready to
        use; that is, it should have a consistent structure, and
        missing data values should be kept to a minimum. The
        observation data must reside in one or more CSV files in an
        Amazon Simple Storage Service (Amazon S3) bucket, along with a
        schema that describes the data items by name and type. The
        same schema must be used for all of the data files referenced
        by the `DataSource`.

        After the `DataSource` has been created, it's ready to use in
        evaluations and batch predictions. If you plan to use the
        `DataSource` to train an `MLModel`, the `DataSource` requires
        another item: a recipe. A recipe describes the observation
        variables that participate in training an `MLModel`. A recipe
        describes how each input variable will be used in training.
        Will the variable be included or excluded from training? Will
        the variable be manipulated, for example, combined with
        another variable, or split apart into word combinations? The
        recipe provides answers to these questions. For more
        information, see the `Amazon Machine Learning Developer
        Guide`_.

        :type data_source_id: string
        :param data_source_id: A user-supplied identifier that uniquely
            identifies the `DataSource`.

        :type data_source_name: string
        :param data_source_name: A user-supplied name or description of the
            `DataSource`.

        :type data_spec: dict
        :param data_spec:
        The data specification of a `DataSource`:


        + DataLocationS3 - Amazon Simple Storage Service (Amazon S3) location
              of the observation data.
        + DataSchemaLocationS3 - Amazon S3 location of the `DataSchema`.
        + DataSchema - A JSON string representing the schema. This is not
              required if `DataSchemaUri` is specified.
        + DataRearrangement - A JSON string representing the splitting
              requirement of a `Datasource`. Sample - ` "{\"randomSeed\":\"some-
              random-seed\",
              \"splitting\":{\"percentBegin\":10,\"percentEnd\":60}}"`

        :type compute_statistics: boolean
        :param compute_statistics: The compute statistics for a `DataSource`.
            The statistics are generated from the observation data referenced
            by a `DataSource`. Amazon ML uses the statistics internally during
            an `MLModel` training. This parameter must be set to `True` if the
            ``DataSource `` needs to be used for `MLModel` training

        """
        params = {
            'DataSourceId': data_source_id,
            'DataSpec': data_spec,
        }
        if data_source_name is not None:
            params['DataSourceName'] = data_source_name
        if compute_statistics is not None:
            params['ComputeStatistics'] = compute_statistics
        return self.make_request(action='CreateDataSourceFromS3',
                                 body=json.dumps(params))

    def create_evaluation(self, evaluation_id, ml_model_id,
                          evaluation_data_source_id, evaluation_name=None):
        """
        Creates a new `Evaluation` of an `MLModel`. An `MLModel` is
        evaluated on a set of observations associated to a
        `DataSource`. Like a `DataSource` for an `MLModel`, the
        `DataSource` for an `Evaluation` contains values for the
        Target Variable. The `Evaluation` compares the predicted
        result for each observation to the actual outcome and provides
        a summary so that you know how effective the `MLModel`
        functions on the test data. Evaluation generates a relevant
        performance metric such as BinaryAUC, RegressionRMSE or
        MulticlassAvgFScore based on the corresponding `MLModelType`:
        `BINARY`, `REGRESSION` or `MULTICLASS`.

        `CreateEvaluation` is an asynchronous operation. In response
        to `CreateEvaluation`, Amazon Machine Learning (Amazon ML)
        immediately returns and sets the evaluation status to
        `PENDING`. After the `Evaluation` is created and ready for
        use, Amazon ML sets the status to `COMPLETED`.

        You can use the GetEvaluation operation to check progress of
        the evaluation during the creation operation.

        :type evaluation_id: string
        :param evaluation_id: A user-supplied ID that uniquely identifies the
            `Evaluation`.

        :type evaluation_name: string
        :param evaluation_name: A user-supplied name or description of the
            `Evaluation`.

        :type ml_model_id: string
        :param ml_model_id: The ID of the `MLModel` to evaluate.
        The schema used in creating the `MLModel` must match the schema of the
            `DataSource` used in the `Evaluation`.

        :type evaluation_data_source_id: string
        :param evaluation_data_source_id: The ID of the `DataSource` for the
            evaluation. The schema of the `DataSource` must match the schema
            used to create the `MLModel`.

        """
        params = {
            'EvaluationId': evaluation_id,
            'MLModelId': ml_model_id,
            'EvaluationDataSourceId': evaluation_data_source_id,
        }
        if evaluation_name is not None:
            params['EvaluationName'] = evaluation_name
        return self.make_request(action='CreateEvaluation',
                                 body=json.dumps(params))

    def create_ml_model(self, ml_model_id, ml_model_type,
                        training_data_source_id, ml_model_name=None,
                        parameters=None, recipe=None, recipe_uri=None):
        """
        Creates a new `MLModel` using the data files and the recipe as
        information sources.

        An `MLModel` is nearly immutable. Users can only update the
        `MLModelName` and the `ScoreThreshold` in an `MLModel` without
        creating a new `MLModel`.

        `CreateMLModel` is an asynchronous operation. In response to
        `CreateMLModel`, Amazon Machine Learning (Amazon ML)
        immediately returns and sets the `MLModel` status to
        `PENDING`. After the `MLModel` is created and ready for use,
        Amazon ML sets the status to `COMPLETED`.

        You can use the GetMLModel operation to check progress of the
        `MLModel` during the creation operation.

        CreateMLModel requires a `DataSource` with computed
        statistics, which can be created by setting
        `ComputeStatistics` to `True` in CreateDataSourceFromRDS,
        CreateDataSourceFromS3, or CreateDataSourceFromRedshift
        operations.

        :type ml_model_id: string
        :param ml_model_id: A user-supplied ID that uniquely identifies the
            `MLModel`.

        :type ml_model_name: string
        :param ml_model_name: A user-supplied name or description of the
            `MLModel`.

        :type ml_model_type: string
        :param ml_model_type: The category of supervised learning that this
            `MLModel` will address. Choose from the following types:

        + Choose `REGRESSION` if the `MLModel` will be used to predict a
              numeric value.
        + Choose `BINARY` if the `MLModel` result has two possible values.
        + Choose `MULTICLASS` if the `MLModel` result has a limited number of
              values.


        For more information, see the `Amazon Machine Learning Developer
            Guide`_.

        :type parameters: map
        :param parameters:
        A list of the training parameters in the `MLModel`. The list is
            implemented as a map of key/value pairs.

        The following is the current set of training parameters:


        + `sgd.l1RegularizationAmount` - Coefficient regularization L1 norm. It
              controls overfitting the data by penalizing large coefficients.
              This tends to drive coefficients to zero, resulting in sparse
              feature set. If you use this parameter, start by specifying a small
              value such as 1.0E-08. The value is a double that ranges from 0 to
              MAX_DOUBLE. The default is not to use L1 normalization. The
              parameter cannot be used when `L2` is specified. Use this parameter
              sparingly.
        + `sgd.l2RegularizationAmount` - Coefficient regularization L2 norm. It
              controls overfitting the data by penalizing large coefficients.
              This tends to drive coefficients to small, nonzero values. If you
              use this parameter, start by specifying a small value such as
              1.0E-08. The valuseis a double that ranges from 0 to MAX_DOUBLE.
              The default is not to use L2 normalization. This cannot be used
              when `L1` is specified. Use this parameter sparingly.
        + `sgd.maxPasses` - Number of times that the training process traverses
              the observations to build the `MLModel`. The value is an integer
              that ranges from 1 to 10000. The default value is 10.
        + `sgd.maxMLModelSizeInBytes` - Maximum allowed size of the model.
              Depending on the input data, the size of the model might affect its
              performance. The value is an integer that ranges from 100000 to
              2147483648. The default value is 33554432.

        :type training_data_source_id: string
        :param training_data_source_id: The `DataSource` that points to the
            training data.

        :type recipe: string
        :param recipe: The data recipe for creating `MLModel`. You must specify
            either the recipe or its URI. If you dont specify a recipe or its
            URI, Amazon ML creates a default.

        :type recipe_uri: string
        :param recipe_uri: The Amazon Simple Storage Service (Amazon S3)
            location and file name that contains the `MLModel` recipe. You must
            specify either the recipe or its URI. If you dont specify a recipe
            or its URI, Amazon ML creates a default.

        """
        params = {
            'MLModelId': ml_model_id,
            'MLModelType': ml_model_type,
            'TrainingDataSourceId': training_data_source_id,
        }
        if ml_model_name is not None:
            params['MLModelName'] = ml_model_name
        if parameters is not None:
            params['Parameters'] = parameters
        if recipe is not None:
            params['Recipe'] = recipe
        if recipe_uri is not None:
            params['RecipeUri'] = recipe_uri
        return self.make_request(action='CreateMLModel',
                                 body=json.dumps(params))

    def create_realtime_endpoint(self, ml_model_id):
        """
        Creates a real-time endpoint for the `MLModel`. The endpoint
        contains the URI of the `MLModel`; that is, the location to
        send real-time prediction requests for the specified
        `MLModel`.

        :type ml_model_id: string
        :param ml_model_id: The ID assigned to the `MLModel` during creation.

        """
        params = {'MLModelId': ml_model_id, }
        return self.make_request(action='CreateRealtimeEndpoint',
                                 body=json.dumps(params))

    def delete_batch_prediction(self, batch_prediction_id):
        """
        Assigns the DELETED status to a `BatchPrediction`, rendering
        it unusable.

        After using the `DeleteBatchPrediction` operation, you can use
        the GetBatchPrediction operation to verify that the status of
        the `BatchPrediction` changed to DELETED.

        The result of the `DeleteBatchPrediction` operation is
        irreversible.

        :type batch_prediction_id: string
        :param batch_prediction_id: A user-supplied ID that uniquely identifies
            the `BatchPrediction`.

        """
        params = {'BatchPredictionId': batch_prediction_id, }
        return self.make_request(action='DeleteBatchPrediction',
                                 body=json.dumps(params))

    def delete_data_source(self, data_source_id):
        """
        Assigns the DELETED status to a `DataSource`, rendering it
        unusable.

        After using the `DeleteDataSource` operation, you can use the
        GetDataSource operation to verify that the status of the
        `DataSource` changed to DELETED.

        The results of the `DeleteDataSource` operation are
        irreversible.

        :type data_source_id: string
        :param data_source_id: A user-supplied ID that uniquely identifies the
            `DataSource`.

        """
        params = {'DataSourceId': data_source_id, }
        return self.make_request(action='DeleteDataSource',
                                 body=json.dumps(params))

    def delete_evaluation(self, evaluation_id):
        """
        Assigns the `DELETED` status to an `Evaluation`, rendering it
        unusable.

        After invoking the `DeleteEvaluation` operation, you can use
        the GetEvaluation operation to verify that the status of the
        `Evaluation` changed to `DELETED`.

        The results of the `DeleteEvaluation` operation are
        irreversible.

        :type evaluation_id: string
        :param evaluation_id: A user-supplied ID that uniquely identifies the
            `Evaluation` to delete.

        """
        params = {'EvaluationId': evaluation_id, }
        return self.make_request(action='DeleteEvaluation',
                                 body=json.dumps(params))

    def delete_ml_model(self, ml_model_id):
        """
        Assigns the DELETED status to an `MLModel`, rendering it
        unusable.

        After using the `DeleteMLModel` operation, you can use the
        GetMLModel operation to verify that the status of the
        `MLModel` changed to DELETED.

        The result of the `DeleteMLModel` operation is irreversible.

        :type ml_model_id: string
        :param ml_model_id: A user-supplied ID that uniquely identifies the
            `MLModel`.

        """
        params = {'MLModelId': ml_model_id, }
        return self.make_request(action='DeleteMLModel',
                                 body=json.dumps(params))

    def delete_realtime_endpoint(self, ml_model_id):
        """
        Deletes a real time endpoint of an `MLModel`.

        :type ml_model_id: string
        :param ml_model_id: The ID assigned to the `MLModel` during creation.

        """
        params = {'MLModelId': ml_model_id, }
        return self.make_request(action='DeleteRealtimeEndpoint',
                                 body=json.dumps(params))

    def describe_batch_predictions(self, filter_variable=None, eq=None,
                                   gt=None, lt=None, ge=None, le=None,
                                   ne=None, prefix=None, sort_order=None,
                                   next_token=None, limit=None):
        """
        Returns a list of `BatchPrediction` operations that match the
        search criteria in the request.

        :type filter_variable: string
        :param filter_variable:
        Use one of the following variables to filter a list of
            `BatchPrediction`:


        + `CreatedAt` - Sets the search criteria to the `BatchPrediction`
              creation date.
        + `Status` - Sets the search criteria to the `BatchPrediction` status.
        + `Name` - Sets the search criteria to the contents of the
              `BatchPrediction` ** ** `Name`.
        + `IAMUser` - Sets the search criteria to the user account that invoked
              the `BatchPrediction` creation.
        + `MLModelId` - Sets the search criteria to the `MLModel` used in the
              `BatchPrediction`.
        + `DataSourceId` - Sets the search criteria to the `DataSource` used in
              the `BatchPrediction`.
        + `DataURI` - Sets the search criteria to the data file(s) used in the
              `BatchPrediction`. The URL can identify either a file or an Amazon
              Simple Storage Solution (Amazon S3) bucket or directory.

        :type eq: string
        :param eq: The equal to operator. The `BatchPrediction` results will
            have `FilterVariable` values that exactly match the value specified
            with `EQ`.

        :type gt: string
        :param gt: The greater than operator. The `BatchPrediction` results
            will have `FilterVariable` values that are greater than the value
            specified with `GT`.

        :type lt: string
        :param lt: The less than operator. The `BatchPrediction` results will
            have `FilterVariable` values that are less than the value specified
            with `LT`.

        :type ge: string
        :param ge: The greater than or equal to operator. The `BatchPrediction`
            results will have `FilterVariable` values that are greater than or
            equal to the value specified with `GE`.

        :type le: string
        :param le: The less than or equal to operator. The `BatchPrediction`
            results will have `FilterVariable` values that are less than or
            equal to the value specified with `LE`.

        :type ne: string
        :param ne: The not equal to operator. The `BatchPrediction` results
            will have `FilterVariable` values not equal to the value specified
            with `NE`.

        :type prefix: string
        :param prefix:
        A string that is found at the beginning of a variable, such as `Name`
            or `Id`.

        For example, a `Batch Prediction` operation could have the `Name`
            `2014-09-09-HolidayGiftMailer`. To search for this
            `BatchPrediction`, select `Name` for the `FilterVariable` and any
            of the following strings for the `Prefix`:


        + 2014-09
        + 2014-09-09
        + 2014-09-09-Holiday

        :type sort_order: string
        :param sort_order: A two-value parameter that determines the sequence
            of the resulting list of `MLModel`s.

        + `asc` - Arranges the list in ascending order (A-Z, 0-9).
        + `dsc` - Arranges the list in descending order (Z-A, 9-0).


        Results are sorted by `FilterVariable`.

        :type next_token: string
        :param next_token: An ID of the page in the paginated results.

        :type limit: integer
        :param limit: The number of pages of information to include in the
            result. The range of acceptable values is 1 through 100. The
            default value is 100.

        """
        params = {}
        if filter_variable is not None:
            params['FilterVariable'] = filter_variable
        if eq is not None:
            params['EQ'] = eq
        if gt is not None:
            params['GT'] = gt
        if lt is not None:
            params['LT'] = lt
        if ge is not None:
            params['GE'] = ge
        if le is not None:
            params['LE'] = le
        if ne is not None:
            params['NE'] = ne
        if prefix is not None:
            params['Prefix'] = prefix
        if sort_order is not None:
            params['SortOrder'] = sort_order
        if next_token is not None:
            params['NextToken'] = next_token
        if limit is not None:
            params['Limit'] = limit
        return self.make_request(action='DescribeBatchPredictions',
                                 body=json.dumps(params))

    def describe_data_sources(self, filter_variable=None, eq=None, gt=None,
                              lt=None, ge=None, le=None, ne=None,
                              prefix=None, sort_order=None, next_token=None,
                              limit=None):
        """
        Returns a list of `DataSource` that match the search criteria
        in the request.

        :type filter_variable: string
        :param filter_variable:
        Use one of the following variables to filter a list of `DataSource`:


        + `CreatedAt` - Sets the search criteria to `DataSource` creation
              dates.
        + `Status` - Sets the search criteria to `DataSource` statuses.
        + `Name` - Sets the search criteria to the contents of `DataSource` **
              ** `Name`.
        + `DataUri` - Sets the search criteria to the URI of data files used to
              create the `DataSource`. The URI can identify either a file or an
              Amazon Simple Storage Service (Amazon S3) bucket or directory.
        + `IAMUser` - Sets the search criteria to the user account that invoked
              the `DataSource` creation.

        :type eq: string
        :param eq: The equal to operator. The `DataSource` results will have
            `FilterVariable` values that exactly match the value specified with
            `EQ`.

        :type gt: string
        :param gt: The greater than operator. The `DataSource` results will
            have `FilterVariable` values that are greater than the value
            specified with `GT`.

        :type lt: string
        :param lt: The less than operator. The `DataSource` results will have
            `FilterVariable` values that are less than the value specified with
            `LT`.

        :type ge: string
        :param ge: The greater than or equal to operator. The `DataSource`
            results will have `FilterVariable` values that are greater than or
            equal to the value specified with `GE`.

        :type le: string
        :param le: The less than or equal to operator. The `DataSource` results
            will have `FilterVariable` values that are less than or equal to
            the value specified with `LE`.

        :type ne: string
        :param ne: The not equal to operator. The `DataSource` results will
            have `FilterVariable` values not equal to the value specified with
            `NE`.

        :type prefix: string
        :param prefix:
        A string that is found at the beginning of a variable, such as `Name`
            or `Id`.

        For example, a `DataSource` could have the `Name`
            `2014-09-09-HolidayGiftMailer`. To search for this `DataSource`,
            select `Name` for the `FilterVariable` and any of the following
            strings for the `Prefix`:


        + 2014-09
        + 2014-09-09
        + 2014-09-09-Holiday

        :type sort_order: string
        :param sort_order: A two-value parameter that determines the sequence
            of the resulting list of `DataSource`.

        + `asc` - Arranges the list in ascending order (A-Z, 0-9).
        + `dsc` - Arranges the list in descending order (Z-A, 9-0).


        Results are sorted by `FilterVariable`.

        :type next_token: string
        :param next_token: The ID of the page in the paginated results.

        :type limit: integer
        :param limit: The maximum number of `DataSource` to include in the
            result.

        """
        params = {}
        if filter_variable is not None:
            params['FilterVariable'] = filter_variable
        if eq is not None:
            params['EQ'] = eq
        if gt is not None:
            params['GT'] = gt
        if lt is not None:
            params['LT'] = lt
        if ge is not None:
            params['GE'] = ge
        if le is not None:
            params['LE'] = le
        if ne is not None:
            params['NE'] = ne
        if prefix is not None:
            params['Prefix'] = prefix
        if sort_order is not None:
            params['SortOrder'] = sort_order
        if next_token is not None:
            params['NextToken'] = next_token
        if limit is not None:
            params['Limit'] = limit
        return self.make_request(action='DescribeDataSources',
                                 body=json.dumps(params))

    def describe_evaluations(self, filter_variable=None, eq=None, gt=None,
                             lt=None, ge=None, le=None, ne=None, prefix=None,
                             sort_order=None, next_token=None, limit=None):
        """
        Returns a list of `DescribeEvaluations` that match the search
        criteria in the request.

        :type filter_variable: string
        :param filter_variable:
        Use one of the following variable to filter a list of `Evaluation`
            objects:


        + `CreatedAt` - Sets the search criteria to the `Evaluation` creation
              date.
        + `Status` - Sets the search criteria to the `Evaluation` status.
        + `Name` - Sets the search criteria to the contents of `Evaluation` **
              ** `Name`.
        + `IAMUser` - Sets the search criteria to the user account that invoked
              an `Evaluation`.
        + `MLModelId` - Sets the search criteria to the `MLModel` that was
              evaluated.
        + `DataSourceId` - Sets the search criteria to the `DataSource` used in
              `Evaluation`.
        + `DataUri` - Sets the search criteria to the data file(s) used in
              `Evaluation`. The URL can identify either a file or an Amazon
              Simple Storage Solution (Amazon S3) bucket or directory.

        :type eq: string
        :param eq: The equal to operator. The `Evaluation` results will have
            `FilterVariable` values that exactly match the value specified with
            `EQ`.

        :type gt: string
        :param gt: The greater than operator. The `Evaluation` results will
            have `FilterVariable` values that are greater than the value
            specified with `GT`.

        :type lt: string
        :param lt: The less than operator. The `Evaluation` results will have
            `FilterVariable` values that are less than the value specified with
            `LT`.

        :type ge: string
        :param ge: The greater than or equal to operator. The `Evaluation`
            results will have `FilterVariable` values that are greater than or
            equal to the value specified with `GE`.

        :type le: string
        :param le: The less than or equal to operator. The `Evaluation` results
            will have `FilterVariable` values that are less than or equal to
            the value specified with `LE`.

        :type ne: string
        :param ne: The not equal to operator. The `Evaluation` results will
            have `FilterVariable` values not equal to the value specified with
            `NE`.

        :type prefix: string
        :param prefix:
        A string that is found at the beginning of a variable, such as `Name`
            or `Id`.

        For example, an `Evaluation` could have the `Name`
            `2014-09-09-HolidayGiftMailer`. To search for this `Evaluation`,
            select `Name` for the `FilterVariable` and any of the following
            strings for the `Prefix`:


        + 2014-09
        + 2014-09-09
        + 2014-09-09-Holiday

        :type sort_order: string
        :param sort_order: A two-value parameter that determines the sequence
            of the resulting list of `Evaluation`.

        + `asc` - Arranges the list in ascending order (A-Z, 0-9).
        + `dsc` - Arranges the list in descending order (Z-A, 9-0).


        Results are sorted by `FilterVariable`.

        :type next_token: string
        :param next_token: The ID of the page in the paginated results.

        :type limit: integer
        :param limit: The maximum number of `Evaluation` to include in the
            result.

        """
        params = {}
        if filter_variable is not None:
            params['FilterVariable'] = filter_variable
        if eq is not None:
            params['EQ'] = eq
        if gt is not None:
            params['GT'] = gt
        if lt is not None:
            params['LT'] = lt
        if ge is not None:
            params['GE'] = ge
        if le is not None:
            params['LE'] = le
        if ne is not None:
            params['NE'] = ne
        if prefix is not None:
            params['Prefix'] = prefix
        if sort_order is not None:
            params['SortOrder'] = sort_order
        if next_token is not None:
            params['NextToken'] = next_token
        if limit is not None:
            params['Limit'] = limit
        return self.make_request(action='DescribeEvaluations',
                                 body=json.dumps(params))

    def describe_ml_models(self, filter_variable=None, eq=None, gt=None,
                           lt=None, ge=None, le=None, ne=None, prefix=None,
                           sort_order=None, next_token=None, limit=None):
        """
        Returns a list of `MLModel` that match the search criteria in
        the request.

        :type filter_variable: string
        :param filter_variable:
        Use one of the following variables to filter a list of `MLModel`:


        + `CreatedAt` - Sets the search criteria to `MLModel` creation date.
        + `Status` - Sets the search criteria to `MLModel` status.
        + `Name` - Sets the search criteria to the contents of `MLModel` ** **
              `Name`.
        + `IAMUser` - Sets the search criteria to the user account that invoked
              the `MLModel` creation.
        + `TrainingDataSourceId` - Sets the search criteria to the `DataSource`
              used to train one or more `MLModel`.
        + `RealtimeEndpointStatus` - Sets the search criteria to the `MLModel`
              real-time endpoint status.
        + `MLModelType` - Sets the search criteria to `MLModel` type: binary,
              regression, or multi-class.
        + `Algorithm` - Sets the search criteria to the algorithm that the
              `MLModel` uses.
        + `TrainingDataURI` - Sets the search criteria to the data file(s) used
              in training a `MLModel`. The URL can identify either a file or an
              Amazon Simple Storage Service (Amazon S3) bucket or directory.

        :type eq: string
        :param eq: The equal to operator. The `MLModel` results will have
            `FilterVariable` values that exactly match the value specified with
            `EQ`.

        :type gt: string
        :param gt: The greater than operator. The `MLModel` results will have
            `FilterVariable` values that are greater than the value specified
            with `GT`.

        :type lt: string
        :param lt: The less than operator. The `MLModel` results will have
            `FilterVariable` values that are less than the value specified with
            `LT`.

        :type ge: string
        :param ge: The greater than or equal to operator. The `MLModel` results
            will have `FilterVariable` values that are greater than or equal to
            the value specified with `GE`.

        :type le: string
        :param le: The less than or equal to operator. The `MLModel` results
            will have `FilterVariable` values that are less than or equal to
            the value specified with `LE`.

        :type ne: string
        :param ne: The not equal to operator. The `MLModel` results will have
            `FilterVariable` values not equal to the value specified with `NE`.

        :type prefix: string
        :param prefix:
        A string that is found at the beginning of a variable, such as `Name`
            or `Id`.

        For example, an `MLModel` could have the `Name`
            `2014-09-09-HolidayGiftMailer`. To search for this `MLModel`,
            select `Name` for the `FilterVariable` and any of the following
            strings for the `Prefix`:


        + 2014-09
        + 2014-09-09
        + 2014-09-09-Holiday

        :type sort_order: string
        :param sort_order: A two-value parameter that determines the sequence
            of the resulting list of `MLModel`.

        + `asc` - Arranges the list in ascending order (A-Z, 0-9).
        + `dsc` - Arranges the list in descending order (Z-A, 9-0).


        Results are sorted by `FilterVariable`.

        :type next_token: string
        :param next_token: The ID of the page in the paginated results.

        :type limit: integer
        :param limit: The number of pages of information to include in the
            result. The range of acceptable values is 1 through 100. The
            default value is 100.

        """
        params = {}
        if filter_variable is not None:
            params['FilterVariable'] = filter_variable
        if eq is not None:
            params['EQ'] = eq
        if gt is not None:
            params['GT'] = gt
        if lt is not None:
            params['LT'] = lt
        if ge is not None:
            params['GE'] = ge
        if le is not None:
            params['LE'] = le
        if ne is not None:
            params['NE'] = ne
        if prefix is not None:
            params['Prefix'] = prefix
        if sort_order is not None:
            params['SortOrder'] = sort_order
        if next_token is not None:
            params['NextToken'] = next_token
        if limit is not None:
            params['Limit'] = limit
        return self.make_request(action='DescribeMLModels',
                                 body=json.dumps(params))

    def get_batch_prediction(self, batch_prediction_id):
        """
        Returns a `BatchPrediction` that includes detailed metadata,
        status, and data file information for a `Batch Prediction`
        request.

        :type batch_prediction_id: string
        :param batch_prediction_id: An ID assigned to the `BatchPrediction` at
            creation.

        """
        params = {'BatchPredictionId': batch_prediction_id, }
        return self.make_request(action='GetBatchPrediction',
                                 body=json.dumps(params))

    def get_data_source(self, data_source_id, verbose=None):
        """
        Returns a `DataSource` that includes metadata and data file
        information, as well as the current status of the
        `DataSource`.

        `GetDataSource` provides results in normal or verbose format.
        The verbose format adds the schema description and the list of
        files pointed to by the DataSource to the normal format.

        :type data_source_id: string
        :param data_source_id: The ID assigned to the `DataSource` at creation.

        :type verbose: boolean
        :param verbose: Specifies whether the `GetDataSource` operation should
            return `DataSourceSchema`.
        If true, `DataSourceSchema` is returned.

        If false, `DataSourceSchema` is not returned.

        """
        params = {'DataSourceId': data_source_id, }
        if verbose is not None:
            params['Verbose'] = verbose
        return self.make_request(action='GetDataSource',
                                 body=json.dumps(params))

    def get_evaluation(self, evaluation_id):
        """
        Returns an `Evaluation` that includes metadata as well as the
        current status of the `Evaluation`.

        :type evaluation_id: string
        :param evaluation_id: The ID of the `Evaluation` to retrieve. The
            evaluation of each `MLModel` is recorded and cataloged. The ID
            provides the means to access the information.

        """
        params = {'EvaluationId': evaluation_id, }
        return self.make_request(action='GetEvaluation',
                                 body=json.dumps(params))

    def get_ml_model(self, ml_model_id, verbose=None):
        """
        Returns an `MLModel` that includes detailed metadata, and data
        source information as well as the current status of the
        `MLModel`.

        `GetMLModel` provides results in normal or verbose format.

        :type ml_model_id: string
        :param ml_model_id: The ID assigned to the `MLModel` at creation.

        :type verbose: boolean
        :param verbose: Specifies whether the `GetMLModel` operation should
            return `Recipe`.
        If true, `Recipe` is returned.

        If false, `Recipe` is not returned.

        """
        params = {'MLModelId': ml_model_id, }
        if verbose is not None:
            params['Verbose'] = verbose
        return self.make_request(action='GetMLModel',
                                 body=json.dumps(params))

    def predict(self, ml_model_id, record, predict_endpoint):
        """
        Generates a prediction for the observation using the specified
        `MLModel`.


        Not all response parameters will be populated because this is
        dependent on the type of requested model.

        :type ml_model_id: string
        :param ml_model_id: A unique identifier of the `MLModel`.

        :type record: map
        :param record: A map of variable name-value pairs that represent an
            observation.

        :type predict_endpoint: string
        :param predict_endpoint: The endpoint to send the predict request to.

        """
        predict_host =  urlsplit(predict_endpoint).hostname
        if predict_host is None:
            predict_host = predict_endpoint

        params = {
            'MLModelId': ml_model_id,
            'Record': record,
            'PredictEndpoint': predict_host,
        }
        return self.make_request(action='Predict',
                                 body=json.dumps(params),
                                 host=predict_host)

    def update_batch_prediction(self, batch_prediction_id,
                                batch_prediction_name):
        """
        Updates the `BatchPredictionName` of a `BatchPrediction`.

        You can use the GetBatchPrediction operation to view the
        contents of the updated data element.

        :type batch_prediction_id: string
        :param batch_prediction_id: The ID assigned to the `BatchPrediction`
            during creation.

        :type batch_prediction_name: string
        :param batch_prediction_name: A new user-supplied name or description
            of the `BatchPrediction`.

        """
        params = {
            'BatchPredictionId': batch_prediction_id,
            'BatchPredictionName': batch_prediction_name,
        }
        return self.make_request(action='UpdateBatchPrediction',
                                 body=json.dumps(params))

    def update_data_source(self, data_source_id, data_source_name):
        """
        Updates the `DataSourceName` of a `DataSource`.

        You can use the GetDataSource operation to view the contents
        of the updated data element.

        :type data_source_id: string
        :param data_source_id: The ID assigned to the `DataSource` during
            creation.

        :type data_source_name: string
        :param data_source_name: A new user-supplied name or description of the
            `DataSource` that will replace the current description.

        """
        params = {
            'DataSourceId': data_source_id,
            'DataSourceName': data_source_name,
        }
        return self.make_request(action='UpdateDataSource',
                                 body=json.dumps(params))

    def update_evaluation(self, evaluation_id, evaluation_name):
        """
        Updates the `EvaluationName` of an `Evaluation`.

        You can use the GetEvaluation operation to view the contents
        of the updated data element.

        :type evaluation_id: string
        :param evaluation_id: The ID assigned to the `Evaluation` during
            creation.

        :type evaluation_name: string
        :param evaluation_name: A new user-supplied name or description of the
            `Evaluation` that will replace the current content.

        """
        params = {
            'EvaluationId': evaluation_id,
            'EvaluationName': evaluation_name,
        }
        return self.make_request(action='UpdateEvaluation',
                                 body=json.dumps(params))

    def update_ml_model(self, ml_model_id, ml_model_name=None,
                        score_threshold=None):
        """
        Updates the `MLModelName` and the `ScoreThreshold` of an
        `MLModel`.

        You can use the GetMLModel operation to view the contents of
        the updated data element.

        :type ml_model_id: string
        :param ml_model_id: The ID assigned to the `MLModel` during creation.

        :type ml_model_name: string
        :param ml_model_name: A user-supplied name or description of the
            `MLModel`.

        :type score_threshold: float
        :param score_threshold: The `ScoreThreshold` used in binary
            classification `MLModel` that marks the boundary between a positive
            prediction and a negative prediction.
        Output values greater than or equal to the `ScoreThreshold` receive a
            positive result from the `MLModel`, such as `True`. Output values
            less than the `ScoreThreshold` receive a negative response from the
            `MLModel`, such as `False`.

        """
        params = {'MLModelId': ml_model_id, }
        if ml_model_name is not None:
            params['MLModelName'] = ml_model_name
        if score_threshold is not None:
            params['ScoreThreshold'] = score_threshold
        return self.make_request(action='UpdateMLModel',
                                 body=json.dumps(params))

    def make_request(self, action, body, host=None):
        headers = {
            'X-Amz-Target': '%s.%s' % (self.TargetPrefix, action),
            'Host': self.region.endpoint,
            'Content-Type': 'application/x-amz-json-1.1',
            'Content-Length': str(len(body)),
        }
        http_request_kwargs = {
            'method':'POST', 'path':'/', 'auth_path':'/', 'params':{},
            'headers': headers, 'data':body
        }
        if host is not None:
            headers['Host'] = host
            http_request_kwargs['host'] = host
        http_request = self.build_base_http_request(**http_request_kwargs)
        response = self._mexe(http_request, sender=None,
                              override_num_retries=10)
        response_body = response.read().decode('utf-8')
        boto.log.debug(response_body)
        if response.status == 200:
            if response_body:
                return json.loads(response_body)
        else:
            json_body = json.loads(response_body)
            fault_name = json_body.get('__type', None)
            exception_class = self._faults.get(fault_name, self.ResponseError)
            raise exception_class(response.status, response.reason,
                                  body=json_body)




#!/usr/bin/env python
# Copyright (c) 2012 Amazon.com, Inc. or its affiliates.  All Rights Reserved
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#

import base64
from datetime import datetime

from tests.unit import unittest
from tests.unit import AWSMockServiceTestCase

from boto.ec2.autoscale import AutoScaleConnection
from boto.ec2.autoscale.group import AutoScalingGroup
from boto.ec2.autoscale.policy import ScalingPolicy
from boto.ec2.autoscale.tag import Tag

from boto.ec2.blockdevicemapping import EBSBlockDeviceType, BlockDeviceMapping

from boto.ec2.autoscale import launchconfig, LaunchConfiguration


class TestAutoScaleGroup(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def setUp(self):
        super(TestAutoScaleGroup, self).setUp()

    def default_body(self):
        return b"""
            <CreateLaunchConfigurationResponse>
              <ResponseMetadata>
                <RequestId>requestid</RequestId>
              </ResponseMetadata>
            </CreateLaunchConfigurationResponse>
        """

    def test_autoscaling_group_with_termination_policies(self):
        self.set_http_response(status_code=200)
        autoscale = AutoScalingGroup(
            name='foo', launch_config='lauch_config',
            min_size=1, max_size=2,
            termination_policies=['OldestInstance', 'OldestLaunchConfiguration'],
            instance_id='test-id')
        self.service_connection.create_auto_scaling_group(autoscale)
        self.assert_request_parameters({
            'Action': 'CreateAutoScalingGroup',
            'AutoScalingGroupName': 'foo',
            'LaunchConfigurationName': 'lauch_config',
            'MaxSize': 2,
            'MinSize': 1,
            'TerminationPolicies.member.1': 'OldestInstance',
            'TerminationPolicies.member.2': 'OldestLaunchConfiguration',
            'InstanceId': 'test-id',
        }, ignore_params_values=['Version'])

    def test_autoscaling_group_single_vpc_zone_identifier(self):
        self.set_http_response(status_code=200)
        autoscale = AutoScalingGroup(
            name='foo',
            vpc_zone_identifier='vpc_zone_1')
        self.service_connection.create_auto_scaling_group(autoscale)
        self.assert_request_parameters({
            'Action': 'CreateAutoScalingGroup',
            'AutoScalingGroupName': 'foo',
            'VPCZoneIdentifier': 'vpc_zone_1',
        }, ignore_params_values=['MaxSize', 'MinSize', 'LaunchConfigurationName', 'Version'])

    def test_autoscaling_group_vpc_zone_identifier_list(self):
        self.set_http_response(status_code=200)
        autoscale = AutoScalingGroup(
            name='foo',
            vpc_zone_identifier=['vpc_zone_1', 'vpc_zone_2'])
        self.service_connection.create_auto_scaling_group(autoscale)
        self.assert_request_parameters({
            'Action': 'CreateAutoScalingGroup',
            'AutoScalingGroupName': 'foo',
            'VPCZoneIdentifier': 'vpc_zone_1,vpc_zone_2',
        }, ignore_params_values=['MaxSize', 'MinSize', 'LaunchConfigurationName', 'Version'])

    def test_autoscaling_group_vpc_zone_identifier_multi(self):
        self.set_http_response(status_code=200)
        autoscale = AutoScalingGroup(
            name='foo',
            vpc_zone_identifier='vpc_zone_1,vpc_zone_2')
        self.service_connection.create_auto_scaling_group(autoscale)
        self.assert_request_parameters({
            'Action': 'CreateAutoScalingGroup',
            'AutoScalingGroupName': 'foo',
            'VPCZoneIdentifier': 'vpc_zone_1,vpc_zone_2',
        }, ignore_params_values=['MaxSize', 'MinSize', 'LaunchConfigurationName', 'Version'])


class TestAutoScaleGroupHonorCooldown(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def default_body(self):
        return b"""
            <SetDesiredCapacityResponse>
              <ResponseMetadata>
                <RequestId>9fb7e2db-6998-11e2-a985-57c82EXAMPLE</RequestId>
              </ResponseMetadata>
            </SetDesiredCapacityResponse>
        """

    def test_honor_cooldown(self):
        self.set_http_response(status_code=200)
        self.service_connection.set_desired_capacity('foo', 10, True)
        self.assert_request_parameters({
            'Action': 'SetDesiredCapacity',
            'AutoScalingGroupName': 'foo',
            'DesiredCapacity': 10,
            'HonorCooldown': 'true',
        }, ignore_params_values=['Version'])


class TestScheduledGroup(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def setUp(self):
        super(TestScheduledGroup, self).setUp()

    def default_body(self):
        return b"""
            <PutScheduledUpdateGroupActionResponse>
                <ResponseMetadata>
                  <RequestId>requestid</RequestId>
                </ResponseMetadata>
            </PutScheduledUpdateGroupActionResponse>
        """

    def test_scheduled_group_creation(self):
        self.set_http_response(status_code=200)
        self.service_connection.create_scheduled_group_action('foo',
                                                              'scheduled-foo',
                                                              desired_capacity=1,
                                                              start_time=datetime(2013, 1, 1, 22, 55, 31),
                                                              end_time=datetime(2013, 2, 1, 22, 55, 31),
                                                              min_size=1,
                                                              max_size=2,
                                                              recurrence='0 10 * * *')
        self.assert_request_parameters({
            'Action': 'PutScheduledUpdateGroupAction',
            'AutoScalingGroupName': 'foo',
            'ScheduledActionName': 'scheduled-foo',
            'MaxSize': 2,
            'MinSize': 1,
            'DesiredCapacity': 1,
            'EndTime': '2013-02-01T22:55:31',
            'StartTime': '2013-01-01T22:55:31',
            'Recurrence': '0 10 * * *',
        }, ignore_params_values=['Version'])


class TestParseAutoScaleGroupResponse(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def default_body(self):
        return b"""
          <DescribeAutoScalingGroupsResult>
             <AutoScalingGroups>
               <member>
                 <Tags/>
                 <SuspendedProcesses/>
                 <AutoScalingGroupName>test_group</AutoScalingGroupName>
                 <HealthCheckType>EC2</HealthCheckType>
                 <CreatedTime>2012-09-27T20:19:47.082Z</CreatedTime>
                 <EnabledMetrics/>
                 <LaunchConfigurationName>test_launchconfig</LaunchConfigurationName>
                 <Instances>
                   <member>
                     <HealthStatus>Healthy</HealthStatus>
                     <AvailabilityZone>us-east-1a</AvailabilityZone>
                     <InstanceId>i-z118d054</InstanceId>
                     <LaunchConfigurationName>test_launchconfig</LaunchConfigurationName>
                     <LifecycleState>InService</LifecycleState>
                   </member>
                 </Instances>
                 <DesiredCapacity>1</DesiredCapacity>
                 <AvailabilityZones>
                   <member>us-east-1c</member>
                   <member>us-east-1a</member>
                 </AvailabilityZones>
                 <LoadBalancerNames/>
                 <MinSize>1</MinSize>
                 <VPCZoneIdentifier/>
                 <HealthCheckGracePeriod>0</HealthCheckGracePeriod>
                 <DefaultCooldown>300</DefaultCooldown>
                 <AutoScalingGroupARN>myarn</AutoScalingGroupARN>
                 <TerminationPolicies>
                   <member>OldestInstance</member>
                   <member>OldestLaunchConfiguration</member>
                 </TerminationPolicies>
                 <MaxSize>2</MaxSize>
                 <InstanceId>Something</InstanceId>
               </member>
             </AutoScalingGroups>
          </DescribeAutoScalingGroupsResult>
        """

    def test_get_all_groups_is_parsed_correctly(self):
        self.set_http_response(status_code=200)
        response = self.service_connection.get_all_groups(names=['test_group'])
        self.assertEqual(len(response), 1, response)
        as_group = response[0]
        self.assertEqual(as_group.availability_zones, ['us-east-1c', 'us-east-1a'])
        self.assertEqual(as_group.default_cooldown, 300)
        self.assertEqual(as_group.desired_capacity, 1)
        self.assertEqual(as_group.enabled_metrics, [])
        self.assertEqual(as_group.health_check_period, 0)
        self.assertEqual(as_group.health_check_type, 'EC2')
        self.assertEqual(as_group.launch_config_name, 'test_launchconfig')
        self.assertEqual(as_group.load_balancers, [])
        self.assertEqual(as_group.min_size, 1)
        self.assertEqual(as_group.max_size, 2)
        self.assertEqual(as_group.name, 'test_group')
        self.assertEqual(as_group.suspended_processes, [])
        self.assertEqual(as_group.tags, [])
        self.assertEqual(as_group.termination_policies,
                         ['OldestInstance', 'OldestLaunchConfiguration'])
        self.assertEqual(as_group.instance_id, 'Something')


class TestDescribeTerminationPolicies(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def default_body(self):
        return b"""
          <DescribeTerminationPolicyTypesResponse>
            <DescribeTerminationPolicyTypesResult>
              <TerminationPolicyTypes>
                <member>ClosestToNextInstanceHour</member>
                <member>Default</member>
                <member>NewestInstance</member>
                <member>OldestInstance</member>
                <member>OldestLaunchConfiguration</member>
              </TerminationPolicyTypes>
            </DescribeTerminationPolicyTypesResult>
            <ResponseMetadata>
              <RequestId>requestid</RequestId>
            </ResponseMetadata>
          </DescribeTerminationPolicyTypesResponse>
        """

    def test_autoscaling_group_with_termination_policies(self):
        self.set_http_response(status_code=200)
        response = self.service_connection.get_termination_policies()
        self.assertListEqual(
            response,
            ['ClosestToNextInstanceHour', 'Default',
             'NewestInstance', 'OldestInstance', 'OldestLaunchConfiguration'])


class TestLaunchConfigurationDescribe(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def default_body(self):
        # This is a dummy response
        return b"""
        <DescribeLaunchConfigurationsResponse>
          <DescribeLaunchConfigurationsResult>
            <LaunchConfigurations>
              <member>
                <AssociatePublicIpAddress>true</AssociatePublicIpAddress>
                <SecurityGroups/>
                <CreatedTime>2013-01-21T23:04:42.200Z</CreatedTime>
                <KernelId/>
                <LaunchConfigurationName>my-test-lc</LaunchConfigurationName>
                <UserData/>
                <InstanceType>m1.small</InstanceType>
                <LaunchConfigurationARN>arn:aws:autoscaling:us-east-1:803981987763:launchConfiguration:9dbbbf87-6141-428a-a409-0752edbe6cad:launchConfigurationName/my-test-lc</LaunchConfigurationARN>
                <BlockDeviceMappings/>
                <ImageId>ami-514ac838</ImageId>
                <KeyName/>
                <RamdiskId/>
                <InstanceMonitoring>
                  <Enabled>true</Enabled>
                </InstanceMonitoring>
                <EbsOptimized>false</EbsOptimized>
                <ClassicLinkVPCId>vpc-12345</ClassicLinkVPCId>
                <ClassicLinkVPCSecurityGroups>
                    <member>sg-1234</member>
                </ClassicLinkVPCSecurityGroups>
              </member>
            </LaunchConfigurations>
          </DescribeLaunchConfigurationsResult>
          <ResponseMetadata>
            <RequestId>d05a22f8-b690-11e2-bf8e-2113fEXAMPLE</RequestId>
          </ResponseMetadata>
        </DescribeLaunchConfigurationsResponse>
        """

    def test_get_all_launch_configurations(self):
        self.set_http_response(status_code=200)

        response = self.service_connection.get_all_launch_configurations()
        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 1)
        self.assertTrue(isinstance(response[0], LaunchConfiguration))

        self.assertEqual(response[0].associate_public_ip_address, True)
        self.assertEqual(response[0].name, "my-test-lc")
        self.assertEqual(response[0].instance_type, "m1.small")
        self.assertEqual(response[0].launch_configuration_arn, "arn:aws:autoscaling:us-east-1:803981987763:launchConfiguration:9dbbbf87-6141-428a-a409-0752edbe6cad:launchConfigurationName/my-test-lc")
        self.assertEqual(response[0].image_id, "ami-514ac838")
        self.assertTrue(isinstance(response[0].instance_monitoring, launchconfig.InstanceMonitoring))
        self.assertEqual(response[0].instance_monitoring.enabled, 'true')
        self.assertEqual(response[0].ebs_optimized, False)
        self.assertEqual(response[0].block_device_mappings, [])
        self.assertEqual(response[0].classic_link_vpc_id, 'vpc-12345')
        self.assertEqual(response[0].classic_link_vpc_security_groups,
                         ['sg-1234'])

        self.assert_request_parameters({
            'Action': 'DescribeLaunchConfigurations',
        }, ignore_params_values=['Version'])

    def test_get_all_configuration_limited(self):
        self.set_http_response(status_code=200)

        response = self.service_connection.get_all_launch_configurations(max_records=10, names=["my-test1", "my-test2"])
        self.assert_request_parameters({
            'Action': 'DescribeLaunchConfigurations',
            'MaxRecords': 10,
            'LaunchConfigurationNames.member.1': 'my-test1',
            'LaunchConfigurationNames.member.2': 'my-test2'
        }, ignore_params_values=['Version'])


class TestLaunchConfiguration(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def default_body(self):
        # This is a dummy response
        return b"""
        <DescribeLaunchConfigurationsResponse>
        </DescribeLaunchConfigurationsResponse>
        """

    def test_launch_config(self):
        # This unit test is based on #753 and #1343
        self.set_http_response(status_code=200)
        dev_sdf = EBSBlockDeviceType(snapshot_id='snap-12345')

        bdm = BlockDeviceMapping()
        bdm['/dev/sdf'] = dev_sdf

        lc = launchconfig.LaunchConfiguration(
            connection=self.service_connection,
            name='launch_config',
            image_id='123456',
            instance_type='m1.large',
            user_data='#!/bin/bash',
            security_groups=['group1'],
            spot_price='price',
            block_device_mappings=[bdm],
            associate_public_ip_address=True,
            volume_type='atype',
            delete_on_termination=False,
            iops=3000,
            classic_link_vpc_id='vpc-1234',
            classic_link_vpc_security_groups=['classic_link_group']
        )

        response = self.service_connection.create_launch_configuration(lc)

        self.assert_request_parameters({
            'Action': 'CreateLaunchConfiguration',
            'BlockDeviceMappings.member.1.DeviceName': '/dev/sdf',
            'BlockDeviceMappings.member.1.Ebs.DeleteOnTermination': 'false',
            'BlockDeviceMappings.member.1.Ebs.SnapshotId': 'snap-12345',
            'EbsOptimized': 'false',
            'LaunchConfigurationName': 'launch_config',
            'ImageId': '123456',
            'UserData': base64.b64encode(b'#!/bin/bash').decode('utf-8'),
            'InstanceMonitoring.Enabled': 'false',
            'InstanceType': 'm1.large',
            'SecurityGroups.member.1': 'group1',
            'SpotPrice': 'price',
            'AssociatePublicIpAddress': 'true',
            'VolumeType': 'atype',
            'DeleteOnTermination': 'false',
            'Iops': 3000,
            'ClassicLinkVPCId': 'vpc-1234',
            'ClassicLinkVPCSecurityGroups.member.1': 'classic_link_group'
        }, ignore_params_values=['Version'])


class TestCreateAutoScalePolicy(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def setUp(self):
        super(TestCreateAutoScalePolicy, self).setUp()

    def default_body(self):
        return b"""
            <PutScalingPolicyResponse xmlns="http://autoscaling.amazonaws.com\
            /doc/2011-01-01/">
              <PutScalingPolicyResult>
                <PolicyARN>arn:aws:autoscaling:us-east-1:803981987763:scaling\
                Policy:b0dcf5e8
            -02e6-4e31-9719-0675d0dc31ae:autoScalingGroupName/my-test-asg:\
            policyName/my-scal
            eout-policy</PolicyARN>
              </PutScalingPolicyResult>
              <ResponseMetadata>
                <RequestId>3cfc6fef-c08b-11e2-a697-2922EXAMPLE</RequestId>
              </ResponseMetadata>
            </PutScalingPolicyResponse>
        """

    def test_scaling_policy_with_min_adjustment_step(self):
        self.set_http_response(status_code=200)

        policy = ScalingPolicy(
            name='foo', as_name='bar',
            adjustment_type='PercentChangeInCapacity', scaling_adjustment=50,
            min_adjustment_step=30)
        self.service_connection.create_scaling_policy(policy)

        self.assert_request_parameters({
            'Action': 'PutScalingPolicy',
            'PolicyName': 'foo',
            'AutoScalingGroupName': 'bar',
            'AdjustmentType': 'PercentChangeInCapacity',
            'ScalingAdjustment': 50,
            'MinAdjustmentStep': 30
        }, ignore_params_values=['Version'])

    def test_scaling_policy_with_wrong_adjustment_type(self):
        self.set_http_response(status_code=200)

        policy = ScalingPolicy(
            name='foo', as_name='bar',
            adjustment_type='ChangeInCapacity', scaling_adjustment=50,
            min_adjustment_step=30)
        self.service_connection.create_scaling_policy(policy)

        self.assert_request_parameters({
            'Action': 'PutScalingPolicy',
            'PolicyName': 'foo',
            'AutoScalingGroupName': 'bar',
            'AdjustmentType': 'ChangeInCapacity',
            'ScalingAdjustment': 50
        }, ignore_params_values=['Version'])

    def test_scaling_policy_without_min_adjustment_step(self):
        self.set_http_response(status_code=200)

        policy = ScalingPolicy(
            name='foo', as_name='bar',
            adjustment_type='PercentChangeInCapacity', scaling_adjustment=50)
        self.service_connection.create_scaling_policy(policy)

        self.assert_request_parameters({
            'Action': 'PutScalingPolicy',
            'PolicyName': 'foo',
            'AutoScalingGroupName': 'bar',
            'AdjustmentType': 'PercentChangeInCapacity',
            'ScalingAdjustment': 50
        }, ignore_params_values=['Version'])


class TestPutNotificationConfiguration(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def setUp(self):
        super(TestPutNotificationConfiguration, self).setUp()

    def default_body(self):
        return b"""
            <PutNotificationConfigurationResponse>
              <ResponseMetadata>
                <RequestId>requestid</RequestId>
              </ResponseMetadata>
            </PutNotificationConfigurationResponse>
        """

    def test_autoscaling_group_put_notification_configuration(self):
        self.set_http_response(status_code=200)
        autoscale = AutoScalingGroup(
            name='ana', launch_config='lauch_config',
            min_size=1, max_size=2,
            termination_policies=['OldestInstance', 'OldestLaunchConfiguration'])
        self.service_connection.put_notification_configuration(autoscale, 'arn:aws:sns:us-east-1:19890506:AutoScaling-Up', ['autoscaling:EC2_INSTANCE_LAUNCH'])
        self.assert_request_parameters({
            'Action': 'PutNotificationConfiguration',
            'AutoScalingGroupName': 'ana',
            'NotificationTypes.member.1': 'autoscaling:EC2_INSTANCE_LAUNCH',
            'TopicARN': 'arn:aws:sns:us-east-1:19890506:AutoScaling-Up',
        }, ignore_params_values=['Version'])


class TestDeleteNotificationConfiguration(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def setUp(self):
        super(TestDeleteNotificationConfiguration, self).setUp()

    def default_body(self):
        return b"""
            <DeleteNotificationConfigurationResponse>
              <ResponseMetadata>
                <RequestId>requestid</RequestId>
              </ResponseMetadata>
            </DeleteNotificationConfigurationResponse>
        """

    def test_autoscaling_group_put_notification_configuration(self):
        self.set_http_response(status_code=200)
        autoscale = AutoScalingGroup(
            name='ana', launch_config='lauch_config',
            min_size=1, max_size=2,
            termination_policies=['OldestInstance', 'OldestLaunchConfiguration'])
        self.service_connection.delete_notification_configuration(autoscale, 'arn:aws:sns:us-east-1:19890506:AutoScaling-Up')
        self.assert_request_parameters({
            'Action': 'DeleteNotificationConfiguration',
            'AutoScalingGroupName': 'ana',
            'TopicARN': 'arn:aws:sns:us-east-1:19890506:AutoScaling-Up',
        }, ignore_params_values=['Version'])


class TestAutoScalingTag(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def default_body(self):
        return b"""
        <CreateOrUpdateTagsResponse>
            <ResponseMetadata>
                <RequestId>requestId</RequestId>
            </ResponseMetadata>
        </CreateOrUpdateTagsResponse>
        """

    def test_create_or_update_tags(self):
        self.set_http_response(status_code=200)

        tags = [
            Tag(
                connection=self.service_connection,
                key='alpha',
                value='tango',
                resource_id='sg-00000000',
                resource_type='auto-scaling-group',
                propagate_at_launch=True
            ),
            Tag(
                connection=self.service_connection,
                key='bravo',
                value='sierra',
                resource_id='sg-00000000',
                resource_type='auto-scaling-group',
                propagate_at_launch=False
            )]

        response = self.service_connection.create_or_update_tags(tags)

        self.assert_request_parameters({
            'Action': 'CreateOrUpdateTags',
            'Tags.member.1.ResourceType': 'auto-scaling-group',
            'Tags.member.1.ResourceId': 'sg-00000000',
            'Tags.member.1.Key': 'alpha',
            'Tags.member.1.Value': 'tango',
            'Tags.member.1.PropagateAtLaunch': 'true',
            'Tags.member.2.ResourceType': 'auto-scaling-group',
            'Tags.member.2.ResourceId': 'sg-00000000',
            'Tags.member.2.Key': 'bravo',
            'Tags.member.2.Value': 'sierra',
            'Tags.member.2.PropagateAtLaunch': 'false'
        }, ignore_params_values=['Version'])

    def test_endElement(self):
        for i in [
            ('Key', 'mykey', 'key'),
            ('Value', 'myvalue', 'value'),
            ('ResourceType', 'auto-scaling-group', 'resource_type'),
            ('ResourceId', 'sg-01234567', 'resource_id'),
            ('PropagateAtLaunch', 'true', 'propagate_at_launch')]:
                self.check_tag_attributes_set(i[0], i[1], i[2])

    def check_tag_attributes_set(self, name, value, attr):
        tag = Tag()
        tag.endElement(name, value, None)
        if value == 'true':
            self.assertEqual(getattr(tag, attr), True)
        else:
            self.assertEqual(getattr(tag, attr), value)


class TestAttachInstances(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def setUp(self):
        super(TestAttachInstances, self).setUp()

    def default_body(self):
        return b"""
            <AttachInstancesResponse>
              <ResponseMetadata>
                <RequestId>requestid</RequestId>
              </ResponseMetadata>
            </AttachInstancesResponse>
        """

    def test_attach_instances(self):
        self.set_http_response(status_code=200)
        self.service_connection.attach_instances(
            'autoscale',
            ['inst2', 'inst1', 'inst4']
        )
        self.assert_request_parameters({
            'Action': 'AttachInstances',
            'AutoScalingGroupName': 'autoscale',
            'InstanceIds.member.1': 'inst2',
            'InstanceIds.member.2': 'inst1',
            'InstanceIds.member.3': 'inst4',
        }, ignore_params_values=['Version'])


class TestDetachInstances(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def setUp(self):
        super(TestDetachInstances, self).setUp()

    def default_body(self):
        return b"""
            <DetachInstancesResponse>
              <ResponseMetadata>
                <RequestId>requestid</RequestId>
              </ResponseMetadata>
            </DetachInstancesResponse>
        """

    def test_detach_instances(self):
        self.set_http_response(status_code=200)
        self.service_connection.detach_instances(
            'autoscale',
            ['inst2', 'inst1', 'inst4']
        )
        self.assert_request_parameters({
            'Action': 'DetachInstances',
            'AutoScalingGroupName': 'autoscale',
            'InstanceIds.member.1': 'inst2',
            'InstanceIds.member.2': 'inst1',
            'InstanceIds.member.3': 'inst4',
            'ShouldDecrementDesiredCapacity': 'true',
        }, ignore_params_values=['Version'])

    def test_detach_instances_with_decrement_desired_capacity(self):
        self.set_http_response(status_code=200)
        self.service_connection.detach_instances(
            'autoscale',
            ['inst2', 'inst1', 'inst4'],
            True
        )
        self.assert_request_parameters({
            'Action': 'DetachInstances',
            'AutoScalingGroupName': 'autoscale',
            'InstanceIds.member.1': 'inst2',
            'InstanceIds.member.2': 'inst1',
            'InstanceIds.member.3': 'inst4',
            'ShouldDecrementDesiredCapacity': 'true',
        }, ignore_params_values=['Version'])

    def test_detach_instances_without_decrement_desired_capacity(self):
        self.set_http_response(status_code=200)
        self.service_connection.detach_instances(
            'autoscale',
            ['inst2', 'inst1', 'inst4'],
            False
        )
        self.assert_request_parameters({
            'Action': 'DetachInstances',
            'AutoScalingGroupName': 'autoscale',
            'InstanceIds.member.1': 'inst2',
            'InstanceIds.member.2': 'inst1',
            'InstanceIds.member.3': 'inst4',
            'ShouldDecrementDesiredCapacity': 'false',
        }, ignore_params_values=['Version'])


class TestGetAccountLimits(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def setUp(self):
        super(TestGetAccountLimits, self).setUp()

    def default_body(self):
        return b"""
            <DescribeAccountLimitsAnswer>
              <MaxNumberOfAutoScalingGroups>6</MaxNumberOfAutoScalingGroups>
              <MaxNumberOfLaunchConfigurations>3</MaxNumberOfLaunchConfigurations>
              <ResponseMetadata>
                <RequestId>requestid</RequestId>
              </ResponseMetadata>
            </DescribeAccountLimitsAnswer>
        """

    def test_autoscaling_group_put_notification_configuration(self):
        self.set_http_response(status_code=200)
        limits = self.service_connection.get_account_limits()
        self.assert_request_parameters({
            'Action': 'DescribeAccountLimits',
        }, ignore_params_values=['Version'])
        self.assertEqual(limits.max_autoscaling_groups, 6)
        self.assertEqual(limits.max_launch_configurations, 3)


class TestGetAdjustmentTypes(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def setUp(self):
        super(TestGetAdjustmentTypes, self).setUp()

    def default_body(self):
        return b"""
            <DescribeAdjustmentTypesResponse xmlns="http://autoscaling.amazonaws.com/doc/201-01-01/">
              <DescribeAdjustmentTypesResult>
                <AdjustmentTypes>
                  <member>
                    <AdjustmentType>ChangeInCapacity</AdjustmentType>
                  </member>
                  <member>
                    <AdjustmentType>ExactCapacity</AdjustmentType>
                  </member>
                  <member>
                    <AdjustmentType>PercentChangeInCapacity</AdjustmentType>
                  </member>
                </AdjustmentTypes>
              </DescribeAdjustmentTypesResult>
              <ResponseMetadata>
                <RequestId>requestId</RequestId>
              </ResponseMetadata>
            </DescribeAdjustmentTypesResponse>
        """

    def test_autoscaling_adjustment_types(self):
        self.set_http_response(status_code=200)
        response = self.service_connection.get_all_adjustment_types()
        self.assert_request_parameters({
            'Action': 'DescribeAdjustmentTypes'
        }, ignore_params_values=['Version'])

        self.assertTrue(isinstance(response, list))
        self.assertEqual(response[0].adjustment_type, "ChangeInCapacity")
        self.assertEqual(response[1].adjustment_type, "ExactCapacity")
        self.assertEqual(response[2].adjustment_type, "PercentChangeInCapacity")


class TestLaunchConfigurationDescribeWithBlockDeviceTypes(AWSMockServiceTestCase):
    connection_class = AutoScaleConnection

    def default_body(self):
        # This is a dummy response
        return b"""
        <DescribeLaunchConfigurationsResponse>
          <DescribeLaunchConfigurationsResult>
            <LaunchConfigurations>
              <member>
                <AssociatePublicIpAddress>true</AssociatePublicIpAddress>
                <SecurityGroups/>
                <CreatedTime>2013-01-21T23:04:42.200Z</CreatedTime>
                <KernelId/>
                <LaunchConfigurationName>my-test-lc</LaunchConfigurationName>
                <UserData/>
                <InstanceType>m1.small</InstanceType>
                <LaunchConfigurationARN>arn:aws:autoscaling:us-east-1:803981987763:launchConfiguration:9dbbbf87-6141-428a-a409-0752edbe6cad:launchConfigurationName/my-test-lc</LaunchConfigurationARN>
                <BlockDeviceMappings>
                  <member>
                    <DeviceName>/dev/xvdp</DeviceName>
                    <Ebs>
                      <SnapshotId>snap-1234abcd</SnapshotId>
                      <Iops>1000</Iops>
                      <DeleteOnTermination>true</DeleteOnTermination>
                      <VolumeType>io1</VolumeType>
                      <VolumeSize>100</VolumeSize>
                    </Ebs>
                  </member>
                  <member>
                    <VirtualName>ephemeral1</VirtualName>
                    <DeviceName>/dev/xvdc</DeviceName>
                  </member>
                  <member>
                    <VirtualName>ephemeral0</VirtualName>
                    <DeviceName>/dev/xvdb</DeviceName>
                  </member>
                  <member>
                    <DeviceName>/dev/xvdh</DeviceName>
                    <Ebs>
                      <Iops>2000</Iops>
                      <DeleteOnTermination>false</DeleteOnTermination>
                      <VolumeType>io1</VolumeType>
                      <VolumeSize>200</VolumeSize>
                    </Ebs>
                  </member>
                </BlockDeviceMappings>
                <ImageId>ami-514ac838</ImageId>
                <KeyName/>
                <RamdiskId/>
                <InstanceMonitoring>
                  <Enabled>true</Enabled>
                </InstanceMonitoring>
                <EbsOptimized>false</EbsOptimized>
              </member>
            </LaunchConfigurations>
          </DescribeLaunchConfigurationsResult>
          <ResponseMetadata>
            <RequestId>d05a22f8-b690-11e2-bf8e-2113fEXAMPLE</RequestId>
          </ResponseMetadata>
        </DescribeLaunchConfigurationsResponse>
        """

    def test_get_all_launch_configurations_with_block_device_types(self):
        self.set_http_response(status_code=200)
        self.service_connection.use_block_device_types = True

        response = self.service_connection.get_all_launch_configurations()
        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 1)
        self.assertTrue(isinstance(response[0], LaunchConfiguration))

        self.assertEqual(response[0].associate_public_ip_address, True)
        self.assertEqual(response[0].name, "my-test-lc")
        self.assertEqual(response[0].instance_type, "m1.small")
        self.assertEqual(response[0].launch_configuration_arn, "arn:aws:autoscaling:us-east-1:803981987763:launchConfiguration:9dbbbf87-6141-428a-a409-0752edbe6cad:launchConfigurationName/my-test-lc")
        self.assertEqual(response[0].image_id, "ami-514ac838")
        self.assertTrue(isinstance(response[0].instance_monitoring, launchconfig.InstanceMonitoring))
        self.assertEqual(response[0].instance_monitoring.enabled, 'true')
        self.assertEqual(response[0].ebs_optimized, False)

        self.assertEqual(response[0].block_device_mappings['/dev/xvdb'].ephemeral_name, 'ephemeral0')

        self.assertEqual(response[0].block_device_mappings['/dev/xvdc'].ephemeral_name, 'ephemeral1')

        self.assertEqual(response[0].block_device_mappings['/dev/xvdp'].snapshot_id, 'snap-1234abcd')
        self.assertEqual(response[0].block_device_mappings['/dev/xvdp'].delete_on_termination, True)
        self.assertEqual(response[0].block_device_mappings['/dev/xvdp'].iops, 1000)
        self.assertEqual(response[0].block_device_mappings['/dev/xvdp'].size, 100)
        self.assertEqual(response[0].block_device_mappings['/dev/xvdp'].volume_type, 'io1')

        self.assertEqual(response[0].block_device_mappings['/dev/xvdh'].delete_on_termination, False)
        self.assertEqual(response[0].block_device_mappings['/dev/xvdh'].iops, 2000)
        self.assertEqual(response[0].block_device_mappings['/dev/xvdh'].size, 200)
        self.assertEqual(response[0].block_device_mappings['/dev/xvdh'].volume_type, 'io1')

        self.assert_request_parameters({
            'Action': 'DescribeLaunchConfigurations',
        }, ignore_params_values=['Version'])

    def test_get_all_configuration_limited(self):
        self.set_http_response(status_code=200)

        response = self.service_connection.get_all_launch_configurations(max_records=10, names=["my-test1", "my-test2"])
        self.assert_request_parameters({
            'Action': 'DescribeLaunchConfigurations',
            'MaxRecords': 10,
            'LaunchConfigurationNames.member.1': 'my-test1',
            'LaunchConfigurationNames.member.2': 'my-test2'
        }, ignore_params_values=['Version'])


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

# Copyright (c) 2011 Mitch Garnaat http://garnaat.org/
# All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

"""
Some unit tests for the S3 MultiDelete
"""

import unittest
import time
from boto.s3.key import Key
from boto.s3.deletemarker import DeleteMarker
from boto.s3.prefix import Prefix
from boto.s3.connection import S3Connection
from boto.exception import S3ResponseError

class S3MultiDeleteTest(unittest.TestCase):
    s3 = True

    def setUp(self):
        self.conn = S3Connection()
        self.bucket_name = 'multidelete-%d' % int(time.time())
        self.bucket = self.conn.create_bucket(self.bucket_name)

    def tearDown(self):
        for key in self.bucket:
            key.delete()
        self.bucket.delete_keys(self.bucket.list_versions())
        self.bucket.delete()

    def test_delete_nothing(self):
        result = self.bucket.delete_keys([])
        self.assertEqual(len(result.deleted), 0)
        self.assertEqual(len(result.errors), 0)

    def test_delete_illegal(self):
        result = self.bucket.delete_keys([{"dict":"notallowed"}])
        self.assertEqual(len(result.deleted), 0)
        self.assertEqual(len(result.errors), 1)

    def test_delete_mix(self):
        result = self.bucket.delete_keys(["king",
                                          ("mice", None),
                                          Key(name="regular"),
                                          Key(),
                                          Prefix(name="folder/"),
                                          DeleteMarker(name="deleted"),
                                          {"bad":"type"}])
        self.assertEqual(len(result.deleted), 4)
        self.assertEqual(len(result.errors), 3)

    def test_delete_quietly(self):
        result = self.bucket.delete_keys(["king"], quiet=True)
        self.assertEqual(len(result.deleted), 0)
        self.assertEqual(len(result.errors), 0)

    def test_delete_must_escape(self):
        result = self.bucket.delete_keys([Key(name=">_<;")])
        self.assertEqual(len(result.deleted), 1)
        self.assertEqual(len(result.errors), 0)

    def test_delete_unknown_version(self):
        no_ver = Key(name="no")
        no_ver.version_id = "version"
        result = self.bucket.delete_keys([no_ver])
        self.assertEqual(len(result.deleted), 0)
        self.assertEqual(len(result.errors), 1)

    def test_delete_kanji(self):
        result = self.bucket.delete_keys([u"漢字", Key(name=u"日本語")])
        self.assertEqual(len(result.deleted), 2)
        self.assertEqual(len(result.errors), 0)

    def test_delete_empty_by_list(self):
        result = self.bucket.delete_keys(self.bucket.list())
        self.assertEqual(len(result.deleted), 0)
        self.assertEqual(len(result.errors), 0)

    def test_delete_kanji_by_list(self):
        for key_name in [u"漢字", u"日本語", u"テスト"]:
            key = self.bucket.new_key(key_name)
            key.set_contents_from_string('this is a test')
        result = self.bucket.delete_keys(self.bucket.list())
        self.assertEqual(len(result.deleted), 3)
        self.assertEqual(len(result.errors), 0)

    def test_delete_with_prefixes(self):
        for key_name in ["a", "a/b", "b"]:
            key = self.bucket.new_key(key_name)
            key.set_contents_from_string('this is a test')

        # First delete all "files": "a" and "b"
        result = self.bucket.delete_keys(self.bucket.list(delimiter="/"))
        self.assertEqual(len(result.deleted), 2)
        # Using delimiter will cause 1 common prefix to be listed
        # which will be skipped as an error.
        self.assertEqual(len(result.errors), 1)
        self.assertEqual(result.errors[0].key, "a/")

        # Next delete any remaining objects: "a/b"
        result = self.bucket.delete_keys(self.bucket.list())
        self.assertEqual(len(result.deleted), 1)
        self.assertEqual(len(result.errors), 0)
        self.assertEqual(result.deleted[0].key, "a/b")

    def test_delete_too_many_versions(self):
        # configure versioning first
        self.bucket.configure_versioning(True)

        # Add 1000 initial versions as DMs by deleting them :-)
        # Adding 1000 objects is painful otherwise...
        key_names = ['key-%03d' % i for i in range(0, 1000)]
        result = self.bucket.delete_keys(key_names)
        self.assertEqual(len(result.deleted) + len(result.errors), 1000)

        # delete them again to create 1000 more delete markers
        result = self.bucket.delete_keys(key_names)
        self.assertEqual(len(result.deleted) + len(result.errors), 1000)

        # Sometimes takes AWS sometime to settle
        time.sleep(10)

        # delete all versions to delete 2000 objects.
        # this tests the 1000 limit.
        result = self.bucket.delete_keys(self.bucket.list_versions())
        self.assertEqual(len(result.deleted) + len(result.errors), 2000)

    def test_1(self):
        nkeys = 100

        # create a bunch of keynames
        key_names = ['key-%03d' % i for i in range(0, nkeys)]

        # create the corresponding keys
        for key_name in key_names:
            key = self.bucket.new_key(key_name)
            key.set_contents_from_string('this is a test')

        # now count keys in bucket
        n = 0
        for key in self.bucket:
            n += 1

        self.assertEqual(n, nkeys)

        # now delete them all
        result = self.bucket.delete_keys(key_names)

        self.assertEqual(len(result.deleted), nkeys)
        self.assertEqual(len(result.errors), 0)

        time.sleep(5)

        # now count keys in bucket
        n = 0
        for key in self.bucket:
            n += 1

        self.assertEqual(n, 0)



# Copyright (c) 2014 Amazon.com, Inc. or its affiliates.
# All Rights Reserved
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

from boto.beanstalk.exception import simple
from tests.compat import unittest


class FakeError(object):
    def __init__(self, code, status, reason, body):
        self.code = code
        self.status = status
        self.reason = reason
        self.body = body


class TestExceptions(unittest.TestCase):
    def test_exception_class_names(self):
        # Create exception from class name
        error = FakeError('TooManyApplications', 400, 'foo', 'bar')
        exception = simple(error)
        self.assertEqual(exception.__class__.__name__, 'TooManyApplications')

        # Create exception from class name + 'Exception' as seen from the
        # live service today
        error = FakeError('TooManyApplicationsException', 400, 'foo', 'bar')
        exception = simple(error)
        self.assertEqual(exception.__class__.__name__, 'TooManyApplications')

        # Make sure message body is present
        self.assertEqual(exception.message, 'bar')



from tests.unit import unittest
from tests.unit import AWSMockServiceTestCase

from boto.cloudfront import CloudFrontConnection
from boto.cloudfront.distribution import Distribution, DistributionConfig, DistributionSummary
from boto.cloudfront.origin import CustomOrigin


class TestCloudFrontConnection(AWSMockServiceTestCase):
    connection_class = CloudFrontConnection

    def setUp(self):
        super(TestCloudFrontConnection, self).setUp()

    def test_get_all_distributions(self):
        body = b"""
        <DistributionList xmlns="http://cloudfront.amazonaws.com/doc/2010-11-01/">
            <Marker></Marker>
            <MaxItems>100</MaxItems>
            <IsTruncated>false</IsTruncated>
            <DistributionSummary>
                <Id>EEEEEEEEEEEEE</Id>
                <Status>InProgress</Status>
                <LastModifiedTime>2014-02-03T11:03:41.087Z</LastModifiedTime>
                <DomainName>abcdef12345678.cloudfront.net</DomainName>
                <CustomOrigin>
                    <DNSName>example.com</DNSName>
                    <HTTPPort>80</HTTPPort>
                    <HTTPSPort>443</HTTPSPort>
                    <OriginProtocolPolicy>http-only</OriginProtocolPolicy>
                </CustomOrigin>
                <CNAME>static.example.com</CNAME>
                <Enabled>true</Enabled>
            </DistributionSummary>
        </DistributionList>
        """
        self.set_http_response(status_code=200, body=body)
        response = self.service_connection.get_all_distributions()

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 1)
        self.assertTrue(isinstance(response[0], DistributionSummary))
        self.assertEqual(response[0].id, "EEEEEEEEEEEEE")
        self.assertEqual(response[0].domain_name, "abcdef12345678.cloudfront.net")
        self.assertEqual(response[0].status, "InProgress")
        self.assertEqual(response[0].cnames, ["static.example.com"])
        self.assertEqual(response[0].enabled, True)
        self.assertTrue(isinstance(response[0].origin, CustomOrigin))
        self.assertEqual(response[0].origin.dns_name, "example.com")
        self.assertEqual(response[0].origin.http_port, 80)
        self.assertEqual(response[0].origin.https_port, 443)
        self.assertEqual(response[0].origin.origin_protocol_policy, 'http-only')

    def test_get_distribution_config(self):
        body = b"""
        <DistributionConfig xmlns="http://cloudfront.amazonaws.com/doc/2010-11-01/">
        <CustomOrigin>
            <DNSName>example.com</DNSName>
            <HTTPPort>80</HTTPPort>
            <HTTPSPort>443</HTTPSPort>
            <OriginProtocolPolicy>http-only</OriginProtocolPolicy>
        </CustomOrigin>
        <CallerReference>1234567890123</CallerReference>
        <CNAME>static.example.com</CNAME>
        <Enabled>true</Enabled>
        </DistributionConfig>
        """

        self.set_http_response(status_code=200, body=body, header={"Etag": "AABBCC"})
        response = self.service_connection.get_distribution_config('EEEEEEEEEEEEE')

        self.assertTrue(isinstance(response, DistributionConfig))
        self.assertTrue(isinstance(response.origin, CustomOrigin))
        self.assertEqual(response.origin.dns_name, "example.com")
        self.assertEqual(response.origin.http_port, 80)
        self.assertEqual(response.origin.https_port, 443)
        self.assertEqual(response.origin.origin_protocol_policy, "http-only")
        self.assertEqual(response.cnames, ["static.example.com"])
        self.assertTrue(response.enabled)
        self.assertEqual(response.etag, "AABBCC")

    def test_set_distribution_config(self):
        get_body = b"""
        <DistributionConfig xmlns="http://cloudfront.amazonaws.com/doc/2010-11-01/">
        <CustomOrigin>
            <DNSName>example.com</DNSName>
            <HTTPPort>80</HTTPPort>
            <HTTPSPort>443</HTTPSPort>
            <OriginProtocolPolicy>http-only</OriginProtocolPolicy>
        </CustomOrigin>
        <CallerReference>1234567890123</CallerReference>
        <CNAME>static.example.com</CNAME>
        <Enabled>true</Enabled>
        </DistributionConfig>
        """

        put_body = b"""
        <Distribution xmlns="http://cloudfront.amazonaws.com/doc/2010-11-01/">
            <Id>EEEEEE</Id>
            <Status>InProgress</Status>
            <LastModifiedTime>2014-02-04T10:47:53.493Z</LastModifiedTime>
            <InProgressInvalidationBatches>0</InProgressInvalidationBatches>
            <DomainName>d2000000000000.cloudfront.net</DomainName>
            <DistributionConfig>
                <CustomOrigin>
                    <DNSName>example.com</DNSName>
                    <HTTPPort>80</HTTPPort>
                    <HTTPSPort>443</HTTPSPort>
                    <OriginProtocolPolicy>match-viewer</OriginProtocolPolicy>
                </CustomOrigin>
                <CallerReference>aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee</CallerReference>
                <Comment>this is a comment</Comment>
                <Enabled>false</Enabled>
            </DistributionConfig>
        </Distribution>
        """

        self.set_http_response(status_code=200, body=get_body, header={"Etag": "AA"})
        conf = self.service_connection.get_distribution_config('EEEEEEE')

        self.set_http_response(status_code=200, body=put_body, header={"Etag": "AABBCCD"})
        conf.comment = 'this is a comment'
        response = self.service_connection.set_distribution_config('EEEEEEE', conf.etag, conf)

        self.assertEqual(response, "AABBCCD")

    def test_get_distribution_info(self):
        body = b"""
        <Distribution xmlns="http://cloudfront.amazonaws.com/doc/2010-11-01/">
            <Id>EEEEEEEEEEEEE</Id>
            <Status>InProgress</Status>
            <LastModifiedTime>2014-02-03T11:03:41.087Z</LastModifiedTime>
            <InProgressInvalidationBatches>0</InProgressInvalidationBatches>
            <DomainName>abcdef12345678.cloudfront.net</DomainName>
            <DistributionConfig>
                <CustomOrigin>
                    <DNSName>example.com</DNSName>
                    <HTTPPort>80</HTTPPort>
                    <HTTPSPort>443</HTTPSPort>
                    <OriginProtocolPolicy>http-only</OriginProtocolPolicy>
                </CustomOrigin>
                <CallerReference>1111111111111</CallerReference>
                <CNAME>static.example.com</CNAME>
                <Enabled>true</Enabled>
            </DistributionConfig>
        </Distribution>
        """

        self.set_http_response(status_code=200, body=body)
        response = self.service_connection.get_distribution_info('EEEEEEEEEEEEE')

        self.assertTrue(isinstance(response, Distribution))
        self.assertTrue(isinstance(response.config, DistributionConfig))
        self.assertTrue(isinstance(response.config.origin, CustomOrigin))
        self.assertEqual(response.config.origin.dns_name, "example.com")
        self.assertEqual(response.config.origin.http_port, 80)
        self.assertEqual(response.config.origin.https_port, 443)
        self.assertEqual(response.config.origin.origin_protocol_policy, "http-only")
        self.assertEqual(response.config.cnames, ["static.example.com"])
        self.assertTrue(response.config.enabled)
        self.assertEqual(response.id, "EEEEEEEEEEEEE")
        self.assertEqual(response.status, "InProgress")
        self.assertEqual(response.domain_name, "abcdef12345678.cloudfront.net")
        self.assertEqual(response.in_progress_invalidation_batches, 0)

    def test_create_distribution(self):
        body = b"""
        <Distribution xmlns="http://cloudfront.amazonaws.com/doc/2010-11-01/">
            <Id>EEEEEEEEEEEEEE</Id>
            <Status>InProgress</Status>
            <LastModifiedTime>2014-02-04T10:34:07.873Z</LastModifiedTime>
            <InProgressInvalidationBatches>0</InProgressInvalidationBatches>
            <DomainName>d2000000000000.cloudfront.net</DomainName>
            <DistributionConfig>
                <CustomOrigin>
                    <DNSName>example.com</DNSName>
                    <HTTPPort>80</HTTPPort>
                    <HTTPSPort>443</HTTPSPort>
                    <OriginProtocolPolicy>match-viewer</OriginProtocolPolicy>
                </CustomOrigin>
                <CallerReference>aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee</CallerReference>
                <Comment>example.com distribution</Comment>
                <Enabled>false</Enabled>
            </DistributionConfig>
        </Distribution>
        """

        self.set_http_response(status_code=201, body=body)
        origin = CustomOrigin("example.com", origin_protocol_policy="match_viewer")
        response = self.service_connection.create_distribution(origin, enabled=False, comment="example.com distribution")

        self.assertTrue(isinstance(response, Distribution))
        self.assertTrue(isinstance(response.config, DistributionConfig))
        self.assertTrue(isinstance(response.config.origin, CustomOrigin))
        self.assertEqual(response.config.origin.dns_name, "example.com")
        self.assertEqual(response.config.origin.http_port, 80)
        self.assertEqual(response.config.origin.https_port, 443)
        self.assertEqual(response.config.origin.origin_protocol_policy, "match-viewer")
        self.assertEqual(response.config.cnames, [])
        self.assertTrue(not response.config.enabled)
        self.assertEqual(response.id, "EEEEEEEEEEEEEE")
        self.assertEqual(response.status, "InProgress")
        self.assertEqual(response.domain_name, "d2000000000000.cloudfront.net")
        self.assertEqual(response.in_progress_invalidation_batches, 0)



# Copyright (c) 2013 Amazon.com, Inc. or its affiliates.  All Rights Reserved
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#

import boto
from boto.compat import json
from boto.connection import AWSQueryConnection
from boto.regioninfo import RegionInfo
from boto.exception import JSONResponseError
from boto.datapipeline import exceptions


class DataPipelineConnection(AWSQueryConnection):
    """
    This is the AWS Data Pipeline API Reference . This guide provides
    descriptions and samples of the AWS Data Pipeline API.

    AWS Data Pipeline is a web service that configures and manages a
    data-driven workflow called a pipeline. AWS Data Pipeline handles
    the details of scheduling and ensuring that data dependencies are
    met so your application can focus on processing the data.

    The AWS Data Pipeline API implements two main sets of
    functionality. The first set of actions configure the pipeline in
    the web service. You call these actions to create a pipeline and
    define data sources, schedules, dependencies, and the transforms
    to be performed on the data.

    The second set of actions are used by a task runner application
    that calls the AWS Data Pipeline API to receive the next task
    ready for processing. The logic for performing the task, such as
    querying the data, running data analysis, or converting the data
    from one format to another, is contained within the task runner.
    The task runner performs the task assigned to it by the web
    service, reporting progress to the web service as it does so. When
    the task is done, the task runner reports the final success or
    failure of the task to the web service.

    AWS Data Pipeline provides an open-source implementation of a task
    runner called AWS Data Pipeline Task Runner. AWS Data Pipeline
    Task Runner provides logic for common data management scenarios,
    such as performing database queries and running data analysis
    using Amazon Elastic MapReduce (Amazon EMR). You can use AWS Data
    Pipeline Task Runner as your task runner, or you can write your
    own task runner to provide custom data management.

    The AWS Data Pipeline API uses the Signature Version 4 protocol
    for signing requests. For more information about how to sign a
    request with this protocol, see `Signature Version 4 Signing
    Process`_. In the code examples in this reference, the Signature
    Version 4 Request parameters are represented as AuthParams.
    """
    APIVersion = "2012-10-29"
    DefaultRegionName = "us-east-1"
    DefaultRegionEndpoint = "datapipeline.us-east-1.amazonaws.com"
    ServiceName = "DataPipeline"
    TargetPrefix = "DataPipeline"
    ResponseError = JSONResponseError

    _faults = {
        "PipelineDeletedException": exceptions.PipelineDeletedException,
        "InvalidRequestException": exceptions.InvalidRequestException,
        "TaskNotFoundException": exceptions.TaskNotFoundException,
        "PipelineNotFoundException": exceptions.PipelineNotFoundException,
        "InternalServiceError": exceptions.InternalServiceError,
    }

    def __init__(self, **kwargs):
        region = kwargs.pop('region', None)
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionEndpoint)
        kwargs['host'] = region.endpoint
        super(DataPipelineConnection, self).__init__(**kwargs)
        self.region = region

    def _required_auth_capability(self):
        return ['hmac-v4']

    def activate_pipeline(self, pipeline_id):
        """
        Validates a pipeline and initiates processing. If the pipeline
        does not pass validation, activation fails.

        Call this action to start processing pipeline tasks of a
        pipeline you've created using the CreatePipeline and
        PutPipelineDefinition actions. A pipeline cannot be modified
        after it has been successfully activated.

        :type pipeline_id: string
        :param pipeline_id: The identifier of the pipeline to activate.

        """
        params = {'pipelineId': pipeline_id, }
        return self.make_request(action='ActivatePipeline',
                                 body=json.dumps(params))

    def create_pipeline(self, name, unique_id, description=None):
        """
        Creates a new empty pipeline. When this action succeeds, you
        can then use the PutPipelineDefinition action to populate the
        pipeline.

        :type name: string
        :param name: The name of the new pipeline. You can use the same name
            for multiple pipelines associated with your AWS account, because
            AWS Data Pipeline assigns each new pipeline a unique pipeline
            identifier.

        :type unique_id: string
        :param unique_id: A unique identifier that you specify. This identifier
            is not the same as the pipeline identifier assigned by AWS Data
            Pipeline. You are responsible for defining the format and ensuring
            the uniqueness of this identifier. You use this parameter to ensure
            idempotency during repeated calls to CreatePipeline. For example,
            if the first call to CreatePipeline does not return a clear
            success, you can pass in the same unique identifier and pipeline
            name combination on a subsequent call to CreatePipeline.
            CreatePipeline ensures that if a pipeline already exists with the
            same name and unique identifier, a new pipeline will not be
            created. Instead, you'll receive the pipeline identifier from the
            previous attempt. The uniqueness of the name and unique identifier
            combination is scoped to the AWS account or IAM user credentials.

        :type description: string
        :param description: The description of the new pipeline.

        """
        params = {'name': name, 'uniqueId': unique_id, }
        if description is not None:
            params['description'] = description
        return self.make_request(action='CreatePipeline',
                                 body=json.dumps(params))

    def delete_pipeline(self, pipeline_id):
        """
        Permanently deletes a pipeline, its pipeline definition and
        its run history. You cannot query or restore a deleted
        pipeline. AWS Data Pipeline will attempt to cancel instances
        associated with the pipeline that are currently being
        processed by task runners. Deleting a pipeline cannot be
        undone.

        To temporarily pause a pipeline instead of deleting it, call
        SetStatus with the status set to Pause on individual
        components. Components that are paused by SetStatus can be
        resumed.

        :type pipeline_id: string
        :param pipeline_id: The identifier of the pipeline to be deleted.

        """
        params = {'pipelineId': pipeline_id, }
        return self.make_request(action='DeletePipeline',
                                 body=json.dumps(params))

    def describe_objects(self, object_ids, pipeline_id, marker=None,
                         evaluate_expressions=None):
        """
        Returns the object definitions for a set of objects associated
        with the pipeline. Object definitions are composed of a set of
        fields that define the properties of the object.

        :type pipeline_id: string
        :param pipeline_id: Identifier of the pipeline that contains the object
            definitions.

        :type object_ids: list
        :param object_ids: Identifiers of the pipeline objects that contain the
            definitions to be described. You can pass as many as 25 identifiers
            in a single call to DescribeObjects.

        :type evaluate_expressions: boolean
        :param evaluate_expressions: Indicates whether any expressions in the
            object should be evaluated when the object descriptions are
            returned.

        :type marker: string
        :param marker: The starting point for the results to be returned. The
            first time you call DescribeObjects, this value should be empty. As
            long as the action returns `HasMoreResults` as `True`, you can call
            DescribeObjects again and pass the marker value from the response
            to retrieve the next set of results.

        """
        params = {
            'pipelineId': pipeline_id,
            'objectIds': object_ids,
        }
        if evaluate_expressions is not None:
            params['evaluateExpressions'] = evaluate_expressions
        if marker is not None:
            params['marker'] = marker
        return self.make_request(action='DescribeObjects',
                                 body=json.dumps(params))

    def describe_pipelines(self, pipeline_ids):
        """
        Retrieve metadata about one or more pipelines. The information
        retrieved includes the name of the pipeline, the pipeline
        identifier, its current state, and the user account that owns
        the pipeline. Using account credentials, you can retrieve
        metadata about pipelines that you or your IAM users have
        created. If you are using an IAM user account, you can
        retrieve metadata about only those pipelines you have read
        permission for.

        To retrieve the full pipeline definition instead of metadata
        about the pipeline, call the GetPipelineDefinition action.

        :type pipeline_ids: list
        :param pipeline_ids: Identifiers of the pipelines to describe. You can
            pass as many as 25 identifiers in a single call to
            DescribePipelines. You can obtain pipeline identifiers by calling
            ListPipelines.

        """
        params = {'pipelineIds': pipeline_ids, }
        return self.make_request(action='DescribePipelines',
                                 body=json.dumps(params))

    def evaluate_expression(self, pipeline_id, expression, object_id):
        """
        Evaluates a string in the context of a specified object. A
        task runner can use this action to evaluate SQL queries stored
        in Amazon S3.

        :type pipeline_id: string
        :param pipeline_id: The identifier of the pipeline.

        :type object_id: string
        :param object_id: The identifier of the object.

        :type expression: string
        :param expression: The expression to evaluate.

        """
        params = {
            'pipelineId': pipeline_id,
            'objectId': object_id,
            'expression': expression,
        }
        return self.make_request(action='EvaluateExpression',
                                 body=json.dumps(params))

    def get_pipeline_definition(self, pipeline_id, version=None):
        """
        Returns the definition of the specified pipeline. You can call
        GetPipelineDefinition to retrieve the pipeline definition you
        provided using PutPipelineDefinition.

        :type pipeline_id: string
        :param pipeline_id: The identifier of the pipeline.

        :type version: string
        :param version: The version of the pipeline definition to retrieve.
            This parameter accepts the values `latest` (default) and `active`.
            Where `latest` indicates the last definition saved to the pipeline
            and `active` indicates the last definition of the pipeline that was
            activated.

        """
        params = {'pipelineId': pipeline_id, }
        if version is not None:
            params['version'] = version
        return self.make_request(action='GetPipelineDefinition',
                                 body=json.dumps(params))

    def list_pipelines(self, marker=None):
        """
        Returns a list of pipeline identifiers for all active
        pipelines. Identifiers are returned only for pipelines you
        have permission to access.

        :type marker: string
        :param marker: The starting point for the results to be returned. The
            first time you call ListPipelines, this value should be empty. As
            long as the action returns `HasMoreResults` as `True`, you can call
            ListPipelines again and pass the marker value from the response to
            retrieve the next set of results.

        """
        params = {}
        if marker is not None:
            params['marker'] = marker
        return self.make_request(action='ListPipelines',
                                 body=json.dumps(params))

    def poll_for_task(self, worker_group, hostname=None,
                      instance_identity=None):
        """
        Task runners call this action to receive a task to perform
        from AWS Data Pipeline. The task runner specifies which tasks
        it can perform by setting a value for the workerGroup
        parameter of the PollForTask call. The task returned by
        PollForTask may come from any of the pipelines that match the
        workerGroup value passed in by the task runner and that was
        launched using the IAM user credentials specified by the task
        runner.

        If tasks are ready in the work queue, PollForTask returns a
        response immediately. If no tasks are available in the queue,
        PollForTask uses long-polling and holds on to a poll
        connection for up to a 90 seconds during which time the first
        newly scheduled task is handed to the task runner. To
        accomodate this, set the socket timeout in your task runner to
        90 seconds. The task runner should not call PollForTask again
        on the same `workerGroup` until it receives a response, and
        this may take up to 90 seconds.

        :type worker_group: string
        :param worker_group: Indicates the type of task the task runner is
            configured to accept and process. The worker group is set as a
            field on objects in the pipeline when they are created. You can
            only specify a single value for `workerGroup` in the call to
            PollForTask. There are no wildcard values permitted in
            `workerGroup`, the string must be an exact, case-sensitive, match.

        :type hostname: string
        :param hostname: The public DNS name of the calling task runner.

        :type instance_identity: dict
        :param instance_identity: Identity information for the Amazon EC2
            instance that is hosting the task runner. You can get this value by
            calling the URI, `http://169.254.169.254/latest/meta-data/instance-
            id`, from the EC2 instance. For more information, go to `Instance
            Metadata`_ in the Amazon Elastic Compute Cloud User Guide. Passing
            in this value proves that your task runner is running on an EC2
            instance, and ensures the proper AWS Data Pipeline service charges
            are applied to your pipeline.

        """
        params = {'workerGroup': worker_group, }
        if hostname is not None:
            params['hostname'] = hostname
        if instance_identity is not None:
            params['instanceIdentity'] = instance_identity
        return self.make_request(action='PollForTask',
                                 body=json.dumps(params))

    def put_pipeline_definition(self, pipeline_objects, pipeline_id):
        """
        Adds tasks, schedules, and preconditions that control the
        behavior of the pipeline. You can use PutPipelineDefinition to
        populate a new pipeline or to update an existing pipeline that
        has not yet been activated.

        PutPipelineDefinition also validates the configuration as it
        adds it to the pipeline. Changes to the pipeline are saved
        unless one of the following three validation errors exists in
        the pipeline.

        #. An object is missing a name or identifier field.
        #. A string or reference field is empty.
        #. The number of objects in the pipeline exceeds the maximum
           allowed objects.



        Pipeline object definitions are passed to the
        PutPipelineDefinition action and returned by the
        GetPipelineDefinition action.

        :type pipeline_id: string
        :param pipeline_id: The identifier of the pipeline to be configured.

        :type pipeline_objects: list
        :param pipeline_objects: The objects that define the pipeline. These
            will overwrite the existing pipeline definition.

        """
        params = {
            'pipelineId': pipeline_id,
            'pipelineObjects': pipeline_objects,
        }
        return self.make_request(action='PutPipelineDefinition',
                                 body=json.dumps(params))

    def query_objects(self, pipeline_id, sphere, marker=None, query=None,
                      limit=None):
        """
        Queries a pipeline for the names of objects that match a
        specified set of conditions.

        The objects returned by QueryObjects are paginated and then
        filtered by the value you set for query. This means the action
        may return an empty result set with a value set for marker. If
        `HasMoreResults` is set to `True`, you should continue to call
        QueryObjects, passing in the returned value for marker, until
        `HasMoreResults` returns `False`.

        :type pipeline_id: string
        :param pipeline_id: Identifier of the pipeline to be queried for object
            names.

        :type query: dict
        :param query: Query that defines the objects to be returned. The Query
            object can contain a maximum of ten selectors. The conditions in
            the query are limited to top-level String fields in the object.
            These filters can be applied to components, instances, and
            attempts.

        :type sphere: string
        :param sphere: Specifies whether the query applies to components or
            instances. Allowable values: `COMPONENT`, `INSTANCE`, `ATTEMPT`.

        :type marker: string
        :param marker: The starting point for the results to be returned. The
            first time you call QueryObjects, this value should be empty. As
            long as the action returns `HasMoreResults` as `True`, you can call
            QueryObjects again and pass the marker value from the response to
            retrieve the next set of results.

        :type limit: integer
        :param limit: Specifies the maximum number of object names that
            QueryObjects will return in a single call. The default value is
            100.

        """
        params = {'pipelineId': pipeline_id, 'sphere': sphere, }
        if query is not None:
            params['query'] = query
        if marker is not None:
            params['marker'] = marker
        if limit is not None:
            params['limit'] = limit
        return self.make_request(action='QueryObjects',
                                 body=json.dumps(params))

    def report_task_progress(self, task_id):
        """
        Updates the AWS Data Pipeline service on the progress of the
        calling task runner. When the task runner is assigned a task,
        it should call ReportTaskProgress to acknowledge that it has
        the task within 2 minutes. If the web service does not recieve
        this acknowledgement within the 2 minute window, it will
        assign the task in a subsequent PollForTask call. After this
        initial acknowledgement, the task runner only needs to report
        progress every 15 minutes to maintain its ownership of the
        task. You can change this reporting time from 15 minutes by
        specifying a `reportProgressTimeout` field in your pipeline.
        If a task runner does not report its status after 5 minutes,
        AWS Data Pipeline will assume that the task runner is unable
        to process the task and will reassign the task in a subsequent
        response to PollForTask. task runners should call
        ReportTaskProgress every 60 seconds.

        :type task_id: string
        :param task_id: Identifier of the task assigned to the task runner.
            This value is provided in the TaskObject that the service returns
            with the response for the PollForTask action.

        """
        params = {'taskId': task_id, }
        return self.make_request(action='ReportTaskProgress',
                                 body=json.dumps(params))

    def report_task_runner_heartbeat(self, taskrunner_id, worker_group=None,
                                     hostname=None):
        """
        Task runners call ReportTaskRunnerHeartbeat every 15 minutes
        to indicate that they are operational. In the case of AWS Data
        Pipeline Task Runner launched on a resource managed by AWS
        Data Pipeline, the web service can use this call to detect
        when the task runner application has failed and restart a new
        instance.

        :type taskrunner_id: string
        :param taskrunner_id: The identifier of the task runner. This value
            should be unique across your AWS account. In the case of AWS Data
            Pipeline Task Runner launched on a resource managed by AWS Data
            Pipeline, the web service provides a unique identifier when it
            launches the application. If you have written a custom task runner,
            you should assign a unique identifier for the task runner.

        :type worker_group: string
        :param worker_group: Indicates the type of task the task runner is
            configured to accept and process. The worker group is set as a
            field on objects in the pipeline when they are created. You can
            only specify a single value for `workerGroup` in the call to
            ReportTaskRunnerHeartbeat. There are no wildcard values permitted
            in `workerGroup`, the string must be an exact, case-sensitive,
            match.

        :type hostname: string
        :param hostname: The public DNS name of the calling task runner.

        """
        params = {'taskrunnerId': taskrunner_id, }
        if worker_group is not None:
            params['workerGroup'] = worker_group
        if hostname is not None:
            params['hostname'] = hostname
        return self.make_request(action='ReportTaskRunnerHeartbeat',
                                 body=json.dumps(params))

    def set_status(self, object_ids, status, pipeline_id):
        """
        Requests that the status of an array of physical or logical
        pipeline objects be updated in the pipeline. This update may
        not occur immediately, but is eventually consistent. The
        status that can be set depends on the type of object.

        :type pipeline_id: string
        :param pipeline_id: Identifies the pipeline that contains the objects.

        :type object_ids: list
        :param object_ids: Identifies an array of objects. The corresponding
            objects can be either physical or components, but not a mix of both
            types.

        :type status: string
        :param status: Specifies the status to be set on all the objects in
            `objectIds`. For components, this can be either `PAUSE` or
            `RESUME`. For instances, this can be either `CANCEL`, `RERUN`, or
            `MARK_FINISHED`.

        """
        params = {
            'pipelineId': pipeline_id,
            'objectIds': object_ids,
            'status': status,
        }
        return self.make_request(action='SetStatus',
                                 body=json.dumps(params))

    def set_task_status(self, task_id, task_status, error_id=None,
                        error_message=None, error_stack_trace=None):
        """
        Notifies AWS Data Pipeline that a task is completed and
        provides information about the final status. The task runner
        calls this action regardless of whether the task was
        sucessful. The task runner does not need to call SetTaskStatus
        for tasks that are canceled by the web service during a call
        to ReportTaskProgress.

        :type task_id: string
        :param task_id: Identifies the task assigned to the task runner. This
            value is set in the TaskObject that is returned by the PollForTask
            action.

        :type task_status: string
        :param task_status: If `FINISHED`, the task successfully completed. If
            `FAILED` the task ended unsuccessfully. The `FALSE` value is used
            by preconditions.

        :type error_id: string
        :param error_id: If an error occurred during the task, this value
            specifies an id value that represents the error. This value is set
            on the physical attempt object. It is used to display error
            information to the user. It should not start with string "Service_"
            which is reserved by the system.

        :type error_message: string
        :param error_message: If an error occurred during the task, this value
            specifies a text description of the error. This value is set on the
            physical attempt object. It is used to display error information to
            the user. The web service does not parse this value.

        :type error_stack_trace: string
        :param error_stack_trace: If an error occurred during the task, this
            value specifies the stack trace associated with the error. This
            value is set on the physical attempt object. It is used to display
            error information to the user. The web service does not parse this
            value.

        """
        params = {'taskId': task_id, 'taskStatus': task_status, }
        if error_id is not None:
            params['errorId'] = error_id
        if error_message is not None:
            params['errorMessage'] = error_message
        if error_stack_trace is not None:
            params['errorStackTrace'] = error_stack_trace
        return self.make_request(action='SetTaskStatus',
                                 body=json.dumps(params))

    def validate_pipeline_definition(self, pipeline_objects, pipeline_id):
        """
        Tests the pipeline definition with a set of validation checks
        to ensure that it is well formed and can run without error.

        :type pipeline_id: string
        :param pipeline_id: Identifies the pipeline whose definition is to be
            validated.

        :type pipeline_objects: list
        :param pipeline_objects: A list of objects that define the pipeline
            changes to validate against the pipeline.

        """
        params = {
            'pipelineId': pipeline_id,
            'pipelineObjects': pipeline_objects,
        }
        return self.make_request(action='ValidatePipelineDefinition',
                                 body=json.dumps(params))

    def make_request(self, action, body):
        headers = {
            'X-Amz-Target': '%s.%s' % (self.TargetPrefix, action),
            'Host': self.region.endpoint,
            'Content-Type': 'application/x-amz-json-1.1',
            'Content-Length': str(len(body)),
        }
        http_request = self.build_base_http_request(
            method='POST', path='/', auth_path='/', params={},
            headers=headers, data=body)
        response = self._mexe(http_request, sender=None,
                              override_num_retries=10)
        response_body = response.read().decode('utf-8')
        boto.log.debug(response_body)
        if response.status == 200:
            if response_body:
                return json.loads(response_body)
        else:
            json_body = json.loads(response_body)
            fault_name = json_body.get('__type', None)
            exception_class = self._faults.get(fault_name, self.ResponseError)
            raise exception_class(response.status, response.reason,
                                  body=json_body)
